@@ -2,7 +2,7 @@
  * Copyright (C) 2004-2005 Lee Hardy <lee -at- leeh.co.uk>
  * Copyright (C) 2004-2005 ircd-ratbox development team
  *
- * $Id: hook.h 18369 2005-01-31 00:16:07Z leeh $
+ * $Id: hook.h 24250 2007-08-22 19:15:08Z androsyn $
  */
 #ifndef INCLUDED_HOOK_H
 #define INCLUDED_HOOK_H
@@ -10,19 +10,19 @@
 typedef struct
 {
 	char *name;
-	dlink_list hooks;
+	rb_dlink_list hooks;
 } hook;
 
 typedef void (*hookfn) (void *data);
 
-int h_iosend_id;
-int h_iorecv_id;
-int h_iorecvctrl_id;
+extern int h_iosend_id;
+extern int h_iorecv_id;
+extern int h_iorecvctrl_id;
 
-int h_burst_client;
-int h_burst_channel;
-int h_burst_finished;
-int h_server_introduced;
+extern int h_burst_client;
+extern int h_burst_channel;
+extern int h_burst_finished;
+extern int h_server_introduced;
 
 void init_hook(void);
 int register_hook(const char *name);
