@@ -3,7 +3,7 @@
  * s_newconf.h: code for dealing with conf stuff
  *
  * Copyright (C) 2004 Lee Hardy <lee@leeh.co.uk>
- * Copyright (C) 2004 ircd-ratbox development team
+ * Copyright (C) 2004-2005 ircd-ratbox development team
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are
@@ -29,35 +29,32 @@
  * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: s_newconf.h 24914 2008-01-11 20:28:57Z androsyn $
+ * $Id: s_newconf.h 26094 2008-09-19 15:33:46Z androsyn $
  */
 
 #ifndef INCLUDED_s_newconf_h
 #define INCLUDED_s_newconf_h
 
-#include "setup.h"
-#include "tools.h"
-
-#ifdef HAVE_LIBCRYPTO
+#ifdef USE_CHALLENGE
 #include <openssl/rsa.h>
 #endif
 
 struct ConfItem;
 
-extern dlink_list cluster_conf_list;
-extern dlink_list shared_conf_list;
-extern dlink_list oper_conf_list;
-extern dlink_list hubleaf_conf_list;
-extern dlink_list server_conf_list;
-extern dlink_list xline_conf_list;
-extern dlink_list resv_conf_list;
-dlink_list tgchange_list;
+extern rb_dlink_list cluster_conf_list;
+extern rb_dlink_list shared_conf_list;
+extern rb_dlink_list oper_conf_list;
+extern rb_dlink_list hubleaf_conf_list;
+extern rb_dlink_list server_conf_list;
+extern rb_dlink_list xline_conf_list;
+extern rb_dlink_list resv_conf_list;
+extern rb_dlink_list tgchange_list;
 
-struct _patricia_tree_t *tgchange_tree;
+extern rb_patricia_tree_t *tgchange_tree;
 
-extern void init_s_newconf(void);
-extern void clear_s_newconf(void);
-extern void clear_s_newconf_bans(void);
+void init_s_newconf(void);
+void clear_s_newconf(void);
+void clear_s_newconf_bans(void);
 
 #define FREE_TARGET(x) ((x)->localClient->targinfo[0])
 #define USED_TARGETS(x) ((x)->localClient->targinfo[1])
@@ -66,8 +63,8 @@ typedef struct
 {
 	char *ip;
 	time_t expiry;
-	patricia_node_t *pnode;
-	dlink_node node;
+	rb_patricia_node_t *pnode;
+	rb_dlink_node node;
 } tgchange;
 
 void add_tgchange(const char *host);
@@ -80,7 +77,7 @@ struct remote_conf
 	char *host;
 	char *server;
 	int flags;
-	dlink_node node;
+	rb_dlink_node node;
 };
 
 /* flags used in shared/cluster */
@@ -114,21 +111,17 @@ struct oper_conf
 	int flags;
 	int umodes;
 
-#ifdef HAVE_LIBCRYPTO
+#ifdef USE_CHALLENGE
 	char *rsa_pubkey_file;
 	RSA *rsa_pubkey;
 #endif
 };
 
-extern struct remote_conf *make_remote_conf(void);
-extern void free_remote_conf(struct remote_conf *);
+struct remote_conf *make_remote_conf(void);
+void free_remote_conf(struct remote_conf *);
 
-extern int find_shared_conf(const char *username, const char *host,
-			const char *server, int flags);
-extern void propagate_generic(struct Client *source_p, const char *command,
-		const char *target, int cap, const char *format, ...);
-extern void cluster_generic(struct Client *, const char *, int cltype,
-			int cap, const char *format, ...);
+int find_shared_conf(const char *username, const char *host, const char *server, int flags);
+void cluster_generic(struct Client *, const char *, int cltype, const char *format, ...);
 
 #define OPER_ENCRYPTED	0x00001
 #define OPER_KLINE	0x00002
@@ -148,8 +141,7 @@ extern void cluster_generic(struct Client *, const char *, int cltype,
 #define OPER_INVIS	0x08000
 #define OPER_SPY	0x10000
 #define OPER_REMOTEBAN	0x20000
-/*			0x40000 	*/
-/* 0x80000 and above are in client.h */
+#define OPER_NEEDSSL	0x40000
 
 #define OPER_FLAGS	(OPER_KLINE|OPER_UNKLINE|OPER_LOCKILL|OPER_GLOBKILL|\
 			 OPER_REMOTE|OPER_GLINE|OPER_XLINE|OPER_RESV|\
@@ -158,33 +150,35 @@ extern void cluster_generic(struct Client *, const char *, int cltype,
 			 OPER_REMOTEBAN)
 
 #define IsOperConfEncrypted(x)	((x)->flags & OPER_ENCRYPTED)
+#define IsOperConfNeedSSL(x)	((x)->flags & OPER_NEEDSSL)
 
-#define IsOperGlobalKill(x)     ((x)->flags2 & OPER_GLOBKILL)
-#define IsOperLocalKill(x)      ((x)->flags2 & OPER_LOCKILL)
-#define IsOperRemote(x)         ((x)->flags2 & OPER_REMOTE)
-#define IsOperUnkline(x)        ((x)->flags2 & OPER_UNKLINE)
-#define IsOperGline(x)          ((x)->flags2 & OPER_GLINE)
-#define IsOperN(x)              ((x)->flags2 & OPER_NICKS)
-#define IsOperK(x)              ((x)->flags2 & OPER_KLINE)
-#define IsOperXline(x)          ((x)->flags2 & OPER_XLINE)
-#define IsOperDie(x)            ((x)->flags2 & OPER_DIE)
-#define IsOperRehash(x)         ((x)->flags2 & OPER_REHASH)
-#define IsOperHiddenAdmin(x)    ((x)->flags2 & OPER_HADMIN)
-#define IsOperAdmin(x)          (((x)->flags2 & OPER_ADMIN) || \
-					((x)->flags2 & OPER_HADMIN))
-#define IsOperOperwall(x)       ((x)->flags2 & OPER_OPERWALL)
-#define IsOperSpy(x)            ((x)->flags2 & OPER_SPY)
-#define IsOperInvis(x)          ((x)->flags2 & OPER_INVIS)
-#define IsOperRemoteBan(x)	((x)->flags2 & OPER_REMOTEBAN)
+#define IsOperGlobalKill(x)     ((x)->operflags & OPER_GLOBKILL)
+#define IsOperLocalKill(x)      ((x)->operflags & OPER_LOCKILL)
+#define IsOperRemote(x)         ((x)->operflags & OPER_REMOTE)
+#define IsOperUnkline(x)        ((x)->operflags & OPER_UNKLINE)
+#define IsOperGline(x)          ((x)->operflags & OPER_GLINE)
+#define IsOperN(x)              ((x)->operflags & OPER_NICKS)
+#define IsOperK(x)              ((x)->operflags & OPER_KLINE)
+#define IsOperXline(x)          ((x)->operflags & OPER_XLINE)
+#define IsOperResv(x)           ((x)->operflags & OPER_RESV)
+#define IsOperDie(x)            ((x)->operflags & OPER_DIE)
+#define IsOperRehash(x)         ((x)->operflags & OPER_REHASH)
+#define IsOperHiddenAdmin(x)    ((x)->operflags & OPER_HADMIN)
+#define IsOperAdmin(x)          (((x)->operflags & OPER_ADMIN) || \
+					((x)->operflags & OPER_HADMIN))
+#define IsOperOperwall(x)       ((x)->operflags & OPER_OPERWALL)
+#define IsOperSpy(x)            ((x)->operflags & OPER_SPY)
+#define IsOperInvis(x)          ((x)->operflags & OPER_INVIS)
+#define IsOperRemoteBan(x)	((x)->operflags & OPER_REMOTEBAN)
 
-extern struct oper_conf *make_oper_conf(void);
-extern void free_oper_conf(struct oper_conf *);
-extern void clear_oper_conf(void);
+struct oper_conf *make_oper_conf(void);
+void free_oper_conf(struct oper_conf *);
+void clear_oper_conf(void);
 
-extern struct oper_conf *find_oper_conf(const char *username, const char *host,
-					const char *locip, const char *oname);
+struct oper_conf *find_oper_conf(const char *username, const char *host,
+				 const char *locip, const char *oname);
 
-extern const char *get_oper_privs(int flags);
+const char *get_oper_privs(int flags);
 
 struct server_conf
 {
@@ -197,13 +191,13 @@ struct server_conf
 	int servers;
 	time_t hold;
 
-	struct irc_sockaddr_storage ipnum;
-	struct irc_sockaddr_storage my_ipnum;
+	struct rb_sockaddr_storage ipnum;
+	struct rb_sockaddr_storage my_ipnum;
 
 	char *class_name;
 	struct Class *class;
-	struct DNSQuery *dns_query;
-	dlink_node node;
+	uint16_t dns_query;
+	rb_dlink_node node;
 
 };
 
@@ -213,6 +207,7 @@ struct server_conf
 #define SERVER_COMPRESSED	0x0008
 #define SERVER_TB		0x0010
 #define SERVER_AUTOCONN		0x0020
+#define SERVER_SSL		0x0040
 
 #define ServerConfIllegal(x)	((x)->flags & SERVER_ILLEGAL)
 #define ServerConfVhosted(x)	((x)->flags & SERVER_VHOSTED)
@@ -220,42 +215,42 @@ struct server_conf
 #define ServerConfCompressed(x)	((x)->flags & SERVER_COMPRESSED)
 #define ServerConfTb(x)		((x)->flags & SERVER_TB)
 #define ServerConfAutoconn(x)	((x)->flags & SERVER_AUTOCONN)
-
-extern struct server_conf *make_server_conf(void);
-extern void free_server_conf(struct server_conf *);
-extern void clear_server_conf(void);
-extern void add_server_conf(struct server_conf *);
-
-extern struct server_conf *find_server_conf(const char *name);
-
-extern void attach_server_conf(struct Client *, struct server_conf *);
-extern void detach_server_conf(struct Client *);
-extern void set_server_conf_autoconn(struct Client *source_p, char *name, 
-					int newval);
+#define ServerConfSSL(x)	((x)->flags & SERVER_SSL)
 
 
-extern struct ConfItem *find_xline(const char *, int);
-extern struct ConfItem *find_xline_mask(const char *);
-extern struct ConfItem *find_nick_resv(const char *name);
+struct server_conf *make_server_conf(void);
+void free_server_conf(struct server_conf *);
+void clear_server_conf(void);
+void add_server_conf(struct server_conf *);
+
+struct server_conf *find_server_conf(const char *name);
+
+void attach_server_conf(struct Client *, struct server_conf *);
+void detach_server_conf(struct Client *);
+void set_server_conf_autoconn(struct Client *source_p, char *name, int newval);
+
+
+struct ConfItem *find_xline(const char *, int);
+struct ConfItem *find_nick_resv(const char *name);
+struct ConfItem *find_xline_mask(const char *);
 struct ConfItem *find_nick_resv_mask(const char *name);
 
-extern int valid_wild_card_simple(const char *);
-extern int clean_resv_nick(const char *);
+int valid_wild_card_simple(const char *);
+int clean_resv_nick(const char *);
 time_t valid_temp_time(const char *p);
 
 struct nd_entry
 {
-	char name[NICKLEN+1];
+	char name[NICKLEN + 1];
 	time_t expire;
 	unsigned int hashv;
 
-	dlink_node hnode;	/* node in hash */
-	dlink_node lnode;	/* node in ll */
+	rb_dlink_node hnode;	/* node in hash */
+	rb_dlink_node lnode;	/* node in ll */
 };
 
-extern void add_nd_entry(const char *name);
-extern void free_nd_entry(struct nd_entry *);
-extern unsigned long get_nd_count(void);
+void add_nd_entry(const char *name);
+void free_nd_entry(struct nd_entry *);
+unsigned long get_nd_count(void);
 
 #endif
-
