@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
  *  Copyright (C) 1996-2002 Hybrid Development Team
- *  Copyright (C) 2002-2004 ircd-ratbox development team
+ *  Copyright (C) 2002-2007 ircd-ratbox development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,17 +18,15 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: scache.h 18204 2005-01-14 17:12:36Z androsyn $
+ *  $Id: scache.h 24639 2007-11-29 16:02:34Z androsyn $
  */
 
 #ifndef INCLUDED_scache_h
 #define INCLUDED_scache_h
 
-extern void clear_scache_hash_table(void);
-extern const char *find_or_add(const char *name);
-extern void count_scache(size_t *, size_t *);
-
+const char *scache_add(const char *name);
+void count_scache(size_t *, size_t *);
 #endif
