@@ -21,7 +21,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: commio.c 26614 2009-07-09 15:03:14Z androsyn $
+ *  $Id: commio.c 27143 2010-11-04 19:47:38Z androsyn $
  */
 #include <libratbox_config.h>
 #include <ratbox_lib.h>
@@ -1890,25 +1890,6 @@ try_devpoll(void)
 	return -1;
 }
 
-static int
-try_sigio(void)
-{
-	if(!rb_init_netio_sigio())
-	{
-		setselect_handler = rb_setselect_sigio;
-		select_handler = rb_select_sigio;
-		setup_fd_handler = rb_setup_fd_sigio;
-		io_sched_event = rb_sigio_sched_event;
-		io_unsched_event = rb_sigio_unsched_event;
-		io_supports_event = rb_sigio_supports_event;
-		io_init_event = rb_sigio_init_event;
-
-		rb_strlcpy(iotype, "sigio", sizeof(iotype));
-		return 0;
-	}
-	return -1;
-}
-
 static int
 try_poll(void)
 {
@@ -2031,11 +2012,6 @@ rb_init_netio(void)
 			if(!try_devpoll())
 				return;
 		}
-		else if(!strcmp("sigio", ioenv))
-		{
-			if(!try_sigio())
-				return;
-		}
 		else if(!strcmp("select", ioenv))
 		{
 			if(!try_select())
@@ -2057,8 +2033,6 @@ rb_init_netio(void)
 		return;
 	if(!try_devpoll())
 		return;
-	if(!try_sigio())
-		return;
 	if(!try_poll())
 		return;
 	if(!try_win32())
