@@ -18,30 +18,24 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: m_gline.c 21128 2005-11-08 20:41:20Z leeh $
+ *  $Id: m_gline.c 26094 2008-09-19 15:33:46Z androsyn $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "struct.h"
 #include "s_gline.h"
-#include "channel.h"
 #include "client.h"
-#include "common.h"
-#include "config.h"
-#include "irc_string.h"
-#include "sprintf_irc.h"
+#include "match.h"
 #include "ircd.h"
 #include "hostmask.h"
 #include "numeric.h"
-#include "commio.h"
 #include "s_conf.h"
 #include "s_newconf.h"
 #include "scache.h"
 #include "send.h"
-#include "msg.h"
 #include "s_serv.h"
 #include "hash.h"
 #include "parse.h"
@@ -53,28 +47,47 @@ static int mc_gline(struct Client *, struct Client *, int, const char **);
 static int ms_gline(struct Client *, struct Client *, int, const char **);
 static int mo_ungline(struct Client *, struct Client *, int, const char **);
 
+static int modinit(void);
+static void moddeinit(void);
+
 struct Message gline_msgtab = {
 	"GLINE", 0, 0, 0, MFLG_SLOW,
 	{mg_unreg, mg_not_oper, {mc_gline, 3}, {ms_gline, 7}, mg_ignore, {mo_gline, 3}}
 };
+
 struct Message ungline_msgtab = {
 	"UNGLINE", 0, 0, 0, MFLG_SLOW,
 	{mg_unreg, mg_not_oper, mg_ignore, mg_ignore, mg_ignore, {mo_ungline, 2}}
 };
 
+
 mapi_clist_av1 gline_clist[] = { &gline_msgtab, &ungline_msgtab, NULL };
-DECLARE_MODULE_AV1(gline, NULL, NULL, gline_clist, NULL, NULL, "$Revision: 21128 $");
+
+DECLARE_MODULE_AV1(gline, modinit, moddeinit, gline_clist, NULL, NULL, "$Revision: 26094 $");
 
 static int majority_gline(struct Client *source_p, const char *user,
 			  const char *host, const char *reason);
-static void set_local_gline(struct Client *source_p, const char *user,
-			    const char *host, const char *reason);
 
 static int check_wild_gline(const char *, const char *);
-static int invalid_gline(struct Client *, const char *, const char *, char *);
+static int invalid_gline(struct Client *, const char *, char *);
 
 static int remove_temp_gline(const char *, const char *);
+static void expire_pending_glines(void *unused);
+static struct ev_entry *pending_gline_ev;
 
+static int
+modinit(void)
+{
+	pending_gline_ev = rb_event_addish("expire_pending_glines", expire_pending_glines, NULL,
+					   CLEANUP_GLINES_TIME);
+	return 0;
+}
+
+static void
+moddeinit(void)
+{
+	rb_event_delete(pending_gline_ev);
+}
 
 /* mo_gline()
  *
@@ -93,15 +106,13 @@ mo_gline(struct Client *client_p, struct Client *source_p, int parc, const char
 
 	if(!ConfigFileEntry.glines)
 	{
-		sendto_one(source_p, ":%s NOTICE %s :GLINE disabled",
-			   me.name, source_p->name);
+		sendto_one_notice(source_p, ":GLINE disabled");
 		return 0;
 	}
 
 	if(!IsOperGline(source_p))
 	{
-		sendto_one(source_p, form_str(ERR_NOPRIVS),
-			   me.name, source_p->name, "gline");
+		sendto_one(source_p, form_str(ERR_NOPRIVS), me.name, source_p->name, "gline");
 		return 0;
 	}
 
@@ -123,9 +134,7 @@ mo_gline(struct Client *client_p, struct Client *source_p, int parc, const char
 		/* ok, its not a host.. abort */
 		if(strchr(parv[1], '.') == NULL)
 		{
-			sendto_one(source_p,
-				   ":%s NOTICE %s :Invalid parameters",
-				   me.name, source_p->name);
+			sendto_one_notice(source_p, ":Invalid parameters");
 			return 0;
 		}
 
@@ -135,18 +144,17 @@ mo_gline(struct Client *client_p, struct Client *source_p, int parc, const char
 
 	reason = LOCAL_COPY(parv[2]);
 
-	if(invalid_gline(source_p, user, host, reason))
+	if(invalid_gline(source_p, user, reason))
 		return 0;
 
 	/* Not enough non-wild characters were found, assume they are trying to gline *@*. */
 	if(check_wild_gline(user, host))
 	{
 		if(MyClient(source_p))
-			sendto_one(source_p,
-				   ":%s NOTICE %s :Please include at least %d non-wildcard "
-				   "characters with the user@host",
-				   me.name, source_p->name, 
-				   ConfigFileEntry.min_nonwildcard);
+			sendto_one_notice(source_p,
+					  ":Please include at least %d non-wildcard "
+					  "characters with the user@host",
+					  ConfigFileEntry.min_nonwildcard);
 		return 0;
 	}
 
@@ -160,18 +168,17 @@ mo_gline(struct Client *client_p, struct Client *source_p, int parc, const char
 		{
 			if(bitlen < ConfigFileEntry.gline_min_cidr)
 			{
-				sendto_one(source_p, ":%s NOTICE %s :Cannot set G-Lines with cidr length < %d",
-					   me.name, source_p->name,
-					   ConfigFileEntry.gline_min_cidr);
+				sendto_one_notice(source_p,
+						  ":Cannot set G-Lines with cidr length < %d",
+						  ConfigFileEntry.gline_min_cidr);
 				return 0;
 			}
 		}
 		/* ipv6 */
 		else if(bitlen < ConfigFileEntry.gline_min_cidr6)
 		{
-			sendto_one(source_p, ":%s NOTICE %s :Cannot set G-Lines with cidr length < %d",
-				   me.name, source_p->name, 
-				   ConfigFileEntry.gline_min_cidr6);
+			sendto_one_notice(source_p, ":Cannot set G-Lines with cidr length < %d",
+					  ConfigFileEntry.gline_min_cidr6);
 			return 0;
 		}
 	}
@@ -180,38 +187,34 @@ mo_gline(struct Client *client_p, struct Client *source_p, int parc, const char
 	 * so already voted comes below gline request --fl
 	 */
 	sendto_realops_flags(UMODE_ALL, L_ALL,
-			"%s!%s@%s on %s is requesting gline for [%s@%s] [%s]",
-			source_p->name, source_p->username,
-			source_p->host, me.name, user, host, reason);
+			     "%s!%s@%s on %s is requesting gline for [%s@%s] [%s]",
+			     source_p->name, source_p->username,
+			     source_p->host, me.name, user, host, reason);
 	ilog(L_GLINE, "R %s %s %s %s %s %s %s",
-	     source_p->name, source_p->username, source_p->host, 
+	     source_p->name, source_p->username, source_p->host,
 	     source_p->servptr->name, user, host, reason);
 
 	/* If at least 3 opers agree this user should be G lined then do it */
 	majority_gline(source_p, user, host, reason);
 
 	/* 4 param version for hyb-7 servers */
-	sendto_server(NULL, NULL, CAP_GLN|CAP_TS6, NOCAPS,
-			":%s GLINE %s %s :%s", 
-			use_id(source_p), user, host, reason);
+	sendto_server(NULL, NULL, CAP_GLN | CAP_TS6, NOCAPS,
+		      ":%s GLINE %s %s :%s", use_id(source_p), user, host, reason);
 	sendto_server(NULL, NULL, CAP_GLN, CAP_TS6,
-			":%s GLINE %s %s :%s", 
-			source_p->name, user, host, reason);
+		      ":%s GLINE %s %s :%s", source_p->name, user, host, reason);
 
 	/* 8 param for hyb-6 */
 	sendto_server(NULL, NULL, NOCAPS, CAP_GLN,
-			":%s GLINE %s %s %s %s %s %s :%s",
-			me.name, source_p->name, source_p->username,
-			source_p->host, source_p->servptr->name, 
-			user, host, reason);
+		      ":%s GLINE %s %s %s %s %s %s :%s",
+		      me.name, source_p->name, source_p->username,
+		      source_p->host, source_p->servptr->name, user, host, reason);
 	return 0;
 }
 
 /* mc_gline()
  */
 static int
-mc_gline(struct Client *client_p, struct Client *source_p,
-	 int parc, const char *parv[])
+mc_gline(struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
 {
 	struct Client *acptr;
 	const char *user;
@@ -229,20 +232,17 @@ mc_gline(struct Client *client_p, struct Client *source_p,
 	host = parv[2];
 	reason = LOCAL_COPY(parv[3]);
 
-	if(invalid_gline(acptr, user, host, reason))
+	if(invalid_gline(acptr, user, reason))
 		return 0;
 
-	sendto_server(client_p, NULL, CAP_GLN|CAP_TS6, NOCAPS,
-		      ":%s GLINE %s %s :%s",
-		      use_id(acptr), user, host, reason);
+	sendto_server(client_p, NULL, CAP_GLN | CAP_TS6, NOCAPS,
+		      ":%s GLINE %s %s :%s", use_id(acptr), user, host, reason);
 	sendto_server(client_p, NULL, CAP_GLN, CAP_TS6,
-		      ":%s GLINE %s %s :%s",
-		      acptr->name, user, host, reason);
+		      ":%s GLINE %s %s :%s", acptr->name, user, host, reason);
 	sendto_server(client_p, NULL, NOCAPS, CAP_GLN,
 		      ":%s GLINE %s %s %s %s %s %s :%s",
-		      acptr->servptr->name, acptr->name, 
-		      acptr->username, acptr->host,
-		      acptr->servptr->name, user, host, reason);
+		      acptr->servptr->name, acptr->name,
+		      acptr->username, acptr->host, acptr->servptr->name, user, host, reason);
 
 	if(!ConfigFileEntry.glines)
 		return 0;
@@ -251,12 +251,11 @@ mc_gline(struct Client *client_p, struct Client *source_p,
 	if(check_wild_gline(user, host))
 	{
 		sendto_realops_flags(UMODE_ALL, L_ALL,
-				"%s!%s@%s on %s is requesting a gline without "
-				"%d non-wildcard characters for [%s@%s] [%s]",
-				acptr->name, acptr->username, 
-				acptr->host, acptr->servptr->name,
-				ConfigFileEntry.min_nonwildcard,
-				user, host, reason);
+				     "%s!%s@%s on %s is requesting a gline without "
+				     "%d non-wildcard characters for [%s@%s] [%s]",
+				     acptr->name, acptr->username,
+				     acptr->host, acptr->servptr->name,
+				     ConfigFileEntry.min_nonwildcard, user, host, reason);
 		return 0;
 	}
 
@@ -270,12 +269,13 @@ mc_gline(struct Client *client_p, struct Client *source_p,
 		{
 			if(bitlen < ConfigFileEntry.gline_min_cidr)
 			{
-				sendto_realops_flags(UMODE_ALL, L_ALL, "%s!%s@%s on %s is requesting a "
+				sendto_realops_flags(UMODE_ALL, L_ALL,
+						     "%s!%s@%s on %s is requesting a "
 						     "gline with a cidr mask < %d for [%s@%s] [%s]",
 						     acptr->name, acptr->username, acptr->host,
 						     acptr->servptr->name,
-						     ConfigFileEntry.gline_min_cidr, 
-						     user, host, reason);
+						     ConfigFileEntry.gline_min_cidr, user, host,
+						     reason);
 				return 0;
 			}
 		}
@@ -286,20 +286,19 @@ mc_gline(struct Client *client_p, struct Client *source_p,
 					     "gline with a cidr mask < %d for [%s@%s] [%s]",
 					     acptr->name, acptr->username, acptr->host,
 					     acptr->servptr->name,
-					     ConfigFileEntry.gline_min_cidr6,
-					     user, host, reason);
+					     ConfigFileEntry.gline_min_cidr6, user, host, reason);
 			return 0;
 		}
 	}
 
 
 	sendto_realops_flags(UMODE_ALL, L_ALL,
-			"%s!%s@%s on %s is requesting gline for [%s@%s] [%s]",
-			acptr->name, acptr->username, acptr->host,
-			acptr->servptr->name, user, host, reason);
+			     "%s!%s@%s on %s is requesting gline for [%s@%s] [%s]",
+			     acptr->name, acptr->username, acptr->host,
+			     acptr->servptr->name, user, host, reason);
 
 	ilog(L_GLINE, "R %s %s %s %s %s %s %s",
-	     source_p->name, source_p->username, source_p->host, 
+	     source_p->name, source_p->username, source_p->host,
 	     source_p->servptr->name, user, host, reason);
 
 	/* If at least 3 opers agree this user should be G lined then do it */
@@ -341,20 +340,17 @@ ms_gline(struct Client *client_p, struct Client *source_p, int parc, const char
 	host = parv[6];
 	reason = LOCAL_COPY(parv[7]);
 
-	if(invalid_gline(acptr, user, host, reason))
+	if(invalid_gline(acptr, user, reason))
 		return 0;
 
-	sendto_server(client_p, NULL, CAP_GLN|CAP_TS6, NOCAPS,
-		      ":%s GLINE %s %s :%s",
-		      use_id(acptr), user, host, reason);
+	sendto_server(client_p, NULL, CAP_GLN | CAP_TS6, NOCAPS,
+		      ":%s GLINE %s %s :%s", use_id(acptr), user, host, reason);
 	sendto_server(client_p, NULL, CAP_GLN, CAP_TS6,
-		      ":%s GLINE %s %s :%s",
-		      acptr->name, user, host, reason);
+		      ":%s GLINE %s %s :%s", acptr->name, user, host, reason);
 	sendto_server(client_p, NULL, NOCAPS, CAP_GLN,
 		      ":%s GLINE %s %s %s %s %s %s :%s",
-		      acptr->servptr->name, acptr->name, 
-		      acptr->username, acptr->host,
-		      acptr->servptr->name, user, host, reason);
+		      acptr->servptr->name, acptr->name,
+		      acptr->username, acptr->host, acptr->servptr->name, user, host, reason);
 
 	if(!ConfigFileEntry.glines)
 		return 0;
@@ -363,23 +359,21 @@ ms_gline(struct Client *client_p, struct Client *source_p, int parc, const char
 	if(check_wild_gline(user, host))
 	{
 		sendto_realops_flags(UMODE_ALL, L_ALL,
-				"%s!%s@%s on %s is requesting a gline without "
-				"%d non-wildcard characters for [%s@%s] [%s]",
-				acptr->name, acptr->username, 
-				acptr->host, acptr->servptr->name,
-				ConfigFileEntry.min_nonwildcard,
-				user, host, reason);
+				     "%s!%s@%s on %s is requesting a gline without "
+				     "%d non-wildcard characters for [%s@%s] [%s]",
+				     acptr->name, acptr->username,
+				     acptr->host, acptr->servptr->name,
+				     ConfigFileEntry.min_nonwildcard, user, host, reason);
 		return 0;
 	}
 
 	sendto_realops_flags(UMODE_ALL, L_ALL,
-			"%s!%s@%s on %s is requesting gline for [%s@%s] [%s]",
-			acptr->name, acptr->username, acptr->host,
-			acptr->servptr->name, user, host, reason);
+			     "%s!%s@%s on %s is requesting gline for [%s@%s] [%s]",
+			     acptr->name, acptr->username, acptr->host,
+			     acptr->servptr->name, user, host, reason);
 
 	ilog(L_GLINE, "R %s %s %s %s %s %s %s",
-	     acptr->name, acptr->username, acptr->host, 
-	     acptr->servptr->name, user, host, reason);
+	     acptr->name, acptr->username, acptr->host, acptr->servptr->name, user, host, reason);
 
 	/* If at least 3 opers agree this user should be G lined then do it */
 	majority_gline(acptr, user, host, reason);
@@ -402,14 +396,13 @@ mo_ungline(struct Client *client_p, struct Client *source_p, int parc, const cha
 
 	if(!ConfigFileEntry.glines)
 	{
-		sendto_one(source_p, ":%s NOTICE %s :UNGLINE disabled", me.name, parv[0]);
+		sendto_one_notice(source_p, ":UNGLINE disabled");
 		return 0;
 	}
 
 	if(!IsOperUnkline(source_p) || !IsOperGline(source_p))
 	{
-		sendto_one(source_p, form_str(ERR_NOPRIVS),
-			   me.name, source_p->name, "unkline");
+		sendto_one(source_p, form_str(ERR_NOPRIVS), me.name, source_p->name, "unkline");
 		return 0;
 	}
 
@@ -420,7 +413,7 @@ mo_ungline(struct Client *client_p, struct Client *source_p, int parc, const cha
 		if(host)
 		{
 			*host++ = '\0';
-	
+
 			/* check for @host */
 			if(*h)
 				user = h;
@@ -438,25 +431,23 @@ mo_ungline(struct Client *client_p, struct Client *source_p, int parc, const cha
 	}
 	else
 	{
-		sendto_one(source_p, ":%s NOTICE %s :Invalid parameters", me.name, parv[0]);
+		sendto_one_notice(source_p, ":Invalid parameters");
 		return 0;
 	}
 
 	if(remove_temp_gline(user, host))
 	{
-		sendto_one(source_p, ":%s NOTICE %s :Un-glined [%s@%s]",
-			   me.name, parv[0], user, host);
+		sendto_one_notice(source_p, ":Un-glined [%s@%s]", user, host);
 		sendto_realops_flags(UMODE_ALL, L_ALL,
 				     "%s has removed the G-Line for: [%s@%s]",
 				     get_oper_name(source_p), user, host);
 		ilog(L_GLINE, "U %s %s %s %s %s %s",
-		     source_p->name, source_p->username, source_p->host, 
+		     source_p->name, source_p->username, source_p->host,
 		     source_p->servptr->name, user, host);
 	}
 	else
 	{
-		sendto_one(source_p, ":%s NOTICE %s :No G-Line for %s@%s",
-			   me.name, parv[0], user, host);
+		sendto_one_notice(source_p, ":No G-Line for %s@%s", user, host);
 	}
 
 	return 0;
@@ -479,7 +470,7 @@ check_wild_gline(const char *user, const char *host)
 	nonwild = 0;
 	p = user;
 
-	while ((tmpch = *p++))
+	while((tmpch = *p++))
 	{
 		if(!IsKWildChar(tmpch))
 		{
@@ -493,7 +484,7 @@ check_wild_gline(const char *user, const char *host)
 	{
 		/* user doesnt, try host */
 		p = host;
-		while ((tmpch = *p++))
+		while((tmpch = *p++))
 		{
 			if(!IsKWildChar(tmpch))
 				if(++nonwild >= ConfigFileEntry.min_nonwildcard)
@@ -514,13 +505,11 @@ check_wild_gline(const char *user, const char *host)
  * side effects -
  */
 static int
-invalid_gline(struct Client *source_p, const char *luser,
-	      const char *lhost, char *lreason)
+invalid_gline(struct Client *source_p, const char *luser, char *lreason)
 {
 	if(strchr(luser, '!'))
 	{
-		sendto_one(source_p, ":%s NOTICE %s :Invalid character '!' in gline",
-			   me.name, source_p->name);
+		sendto_one_notice(source_p, ":Invalid character '!' in gline");
 		return 1;
 	}
 
@@ -530,6 +519,78 @@ invalid_gline(struct Client *source_p, const char *luser,
 	return 0;
 }
 
+/* find_is_glined()
+ * 
+ * inputs       - hostname and username to search for
+ * output       - pointer to struct ConfItem if user@host glined
+ * side effects -
+ */
+static struct ConfItem *
+find_is_glined(const char *host, const char *user)
+{
+	struct ConfItem *aconf;
+	rb_dlink_node *ptr;
+
+	RB_DLINK_FOREACH(ptr, glines.head)
+	{
+		aconf = ptr->data;
+		if((!user || irccmp(aconf->user, user) == 0) &&
+		   (!host || irccmp(aconf->host, host) == 0))
+			return aconf;
+	}
+
+	return NULL;
+}
+
+/* check_glines()
+ *
+ * inputs       -
+ * outputs      -
+ * side effects - all clients will be checked for glines
+ */
+static void
+check_glines(void)
+{
+	struct Client *client_p;
+	struct ConfItem *aconf;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
+
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, lclient_list.head)
+	{
+		client_p = ptr->data;
+
+		if(IsMe(client_p) || !IsClient(client_p))
+			continue;
+
+		if((aconf = find_gline(client_p)) != NULL)
+		{
+			if(IsExemptKline(client_p))
+			{
+				sendto_realops_flags(UMODE_ALL, L_ALL,
+						     "GLINE over-ruled for %s, client is kline_exempt",
+						     get_client_name(client_p, HIDE_IP));
+				continue;
+			}
+
+			if(IsExemptGline(client_p))
+			{
+				sendto_realops_flags(UMODE_ALL, L_ALL,
+						     "GLINE over-ruled for %s, client is gline_exempt",
+						     get_client_name(client_p, HIDE_IP));
+				continue;
+			}
+
+			sendto_realops_flags(UMODE_ALL, L_ALL,
+					     "GLINE active for %s",
+					     get_client_name(client_p, HIDE_IP));
+
+			notify_banned_client(client_p, aconf, G_LINED);
+			continue;
+		}
+	}
+}
+
 /*
  * set_local_gline
  *
@@ -539,8 +600,7 @@ invalid_gline(struct Client *source_p, const char *luser,
  * side effects	-
  */
 static void
-set_local_gline(struct Client *source_p, const char *user,
-		const char *host, const char *reason)
+set_local_gline(struct Client *source_p, const char *user, const char *host, const char *reason)
 {
 	char buffer[IRCD_BUFSIZE];
 	struct ConfItem *aconf;
@@ -548,7 +608,7 @@ set_local_gline(struct Client *source_p, const char *user,
 	char *my_reason;
 	char *oper_reason;
 
-	current_date = smalldate();
+	current_date = smalldate(rb_current_time());
 
 	my_reason = LOCAL_COPY(reason);
 
@@ -557,7 +617,7 @@ set_local_gline(struct Client *source_p, const char *user,
 	aconf->flags |= CONF_FLAGS_TEMPORARY;
 
 	if(strlen(my_reason) > REASONLEN)
-		my_reason[REASONLEN-1] = '\0';
+		my_reason[REASONLEN - 1] = '\0';
 
 	if((oper_reason = strchr(my_reason, '|')) != NULL)
 	{
@@ -565,24 +625,25 @@ set_local_gline(struct Client *source_p, const char *user,
 		oper_reason++;
 
 		if(!EmptyString(oper_reason))
-			DupString(aconf->spasswd, oper_reason);
+			aconf->spasswd = rb_strdup(oper_reason);
 	}
 
-	ircsnprintf(buffer, sizeof(buffer), "%s (%s)", reason, current_date);
+	rb_snprintf(buffer, sizeof(buffer), "%s (%s)", reason, current_date);
 
-	DupString(aconf->passwd, buffer);
-	DupString(aconf->user, user);
-	DupString(aconf->host, host);
-	aconf->hold = CurrentTime + ConfigFileEntry.gline_time;
-	add_gline(aconf);
+	aconf->passwd = rb_strdup(buffer);
+	aconf->user = rb_strdup(user);
+	aconf->host = rb_strdup(host);
+	aconf->hold = rb_current_time() + ConfigFileEntry.gline_time;
+
+	rb_dlinkAddTailAlloc(aconf, &glines);
+	add_conf_by_address(aconf->host, CONF_GLINE, aconf->user, aconf);
 
 	sendto_realops_flags(UMODE_ALL, L_ALL,
 			     "%s!%s@%s on %s has triggered gline for [%s@%s] [%s]",
 			     source_p->name, source_p->username,
-			     source_p->host, source_p->servptr->name,
-			     user, host, reason);
+			     source_p->host, source_p->servptr->name, user, host, reason);
 	ilog(L_GLINE, "T %s %s %s %s %s %s %s",
-	     source_p->name, source_p->username, source_p->host, 
+	     source_p->name, source_p->username, source_p->host,
 	     source_p->servptr->name, user, host, reason);
 
 	check_glines();
@@ -595,28 +656,26 @@ set_local_gline(struct Client *source_p, const char *user,
  * side effects -
  */
 static int
-majority_gline(struct Client *source_p, const char *user,
-	       const char *host, const char *reason)
+majority_gline(struct Client *source_p, const char *user, const char *host, const char *reason)
 {
-	dlink_node *pending_node;
+	rb_dlink_node *pending_node;
 	struct gline_pending *pending;
 
 	/* to avoid desync.. --fl */
-	cleanup_glines(NULL);
+	expire_pending_glines(NULL);
 
 	/* if its already glined, why bother? :) -- fl_ */
 	if(find_is_glined(host, user))
 		return NO;
 
-	DLINK_FOREACH(pending_node, pending_glines.head)
+	RB_DLINK_FOREACH(pending_node, pending_glines.head)
 	{
 		pending = pending_node->data;
 
-		if((irccmp(pending->user, user) == 0) &&
-		   (irccmp(pending->host, host) == 0))
+		if((irccmp(pending->user, user) == 0) && (irccmp(pending->host, host) == 0))
 		{
 			/* check oper or server hasnt already voted */
-			if(((irccmp(pending->oper_user1, source_p->username) == 0) ||
+			if(((irccmp(pending->oper_user1, source_p->username) == 0) &&
 			    (irccmp(pending->oper_host1, source_p->host) == 0)))
 			{
 				sendto_realops_flags(UMODE_ALL, L_ALL, "oper has already voted");
@@ -631,7 +690,7 @@ majority_gline(struct Client *source_p, const char *user,
 			if(pending->oper_user2[0] != '\0')
 			{
 				/* if two other opers on two different servers have voted yes */
-				if(((irccmp(pending->oper_user2, source_p->username) == 0) ||
+				if(((irccmp(pending->oper_user2, source_p->username) == 0) &&
 				    (irccmp(pending->oper_host2, source_p->host) == 0)))
 				{
 					sendto_realops_flags(UMODE_ALL, L_ALL,
@@ -646,51 +705,46 @@ majority_gline(struct Client *source_p, const char *user,
 				}
 
 				/* trigger the gline using the original reason --fl */
-				set_local_gline(source_p, user, host,
-						pending->reason1);
+				set_local_gline(source_p, user, host, pending->reason1);
 
-				cleanup_glines(NULL);
+				expire_pending_glines(NULL);
 				return YES;
 			}
 			else
 			{
-				strlcpy(pending->oper_nick2, source_p->name,
-					sizeof(pending->oper_nick2));
-				strlcpy(pending->oper_user2, source_p->username,
-					sizeof(pending->oper_user2));
-				strlcpy(pending->oper_host2, source_p->host,
-					sizeof(pending->oper_host2));
-				DupString(pending->reason2, reason);
-				pending->oper_server2 = find_or_add(source_p->servptr->name);
-				pending->last_gline_time = CurrentTime;
-				pending->time_request2 = CurrentTime;
+				rb_strlcpy(pending->oper_nick2, source_p->name,
+					   sizeof(pending->oper_nick2));
+				rb_strlcpy(pending->oper_user2, source_p->username,
+					   sizeof(pending->oper_user2));
+				rb_strlcpy(pending->oper_host2, source_p->host,
+					   sizeof(pending->oper_host2));
+				pending->reason2 = rb_strdup(reason);
+				pending->oper_server2 = scache_add(source_p->servptr->name);
+				pending->last_gline_time = rb_current_time();
+				pending->time_request2 = rb_current_time();
 				return NO;
 			}
 		}
 	}
 
 	/* no pending gline, create a new one */
-	pending = (struct gline_pending *) 
-			    MyMalloc(sizeof(struct gline_pending));
+	pending = (struct gline_pending *)rb_malloc(sizeof(struct gline_pending));
 
-	strlcpy(pending->oper_nick1, source_p->name,
-		sizeof(pending->oper_nick1));
-	strlcpy(pending->oper_user1, source_p->username,
-		sizeof(pending->oper_user1));
-	strlcpy(pending->oper_host1, source_p->host,
-		sizeof(pending->oper_host1));
+	rb_strlcpy(pending->oper_nick1, source_p->name, sizeof(pending->oper_nick1));
+	rb_strlcpy(pending->oper_user1, source_p->username, sizeof(pending->oper_user1));
+	rb_strlcpy(pending->oper_host1, source_p->host, sizeof(pending->oper_host1));
 
-	pending->oper_server1 = find_or_add(source_p->servptr->name);
+	pending->oper_server1 = scache_add(source_p->servptr->name);
 
-	strlcpy(pending->user, user, sizeof(pending->user));
-	strlcpy(pending->host, host, sizeof(pending->host));
-	DupString(pending->reason1, reason);
+	rb_strlcpy(pending->user, user, sizeof(pending->user));
+	rb_strlcpy(pending->host, host, sizeof(pending->host));
+	pending->reason1 = rb_strdup(reason);
 	pending->reason2 = NULL;
 
-	pending->last_gline_time = CurrentTime;
-	pending->time_request1 = CurrentTime;
+	pending->last_gline_time = rb_current_time();
+	pending->time_request1 = rb_current_time();
 
-	dlinkAddAlloc(pending, &pending_glines);
+	rb_dlinkAddAlloc(pending, &pending_glines);
 
 	return NO;
 }
@@ -705,14 +759,14 @@ static int
 remove_temp_gline(const char *user, const char *host)
 {
 	struct ConfItem *aconf;
-	dlink_node *ptr;
-	struct irc_sockaddr_storage addr, caddr;
+	rb_dlink_node *ptr;
+	struct rb_sockaddr_storage addr, caddr;
 	int bits, cbits;
 	int mtype, gtype;
 
 	mtype = parse_netmask(host, (struct sockaddr *)&addr, &bits);
 
-	DLINK_FOREACH(ptr, glines.head)
+	RB_DLINK_FOREACH(ptr, glines.head)
 	{
 		aconf = ptr->data;
 
@@ -727,14 +781,47 @@ remove_temp_gline(const char *user, const char *host)
 				continue;
 		}
 		else if(bits != cbits ||
-			!comp_with_mask_sock((struct sockaddr *)&addr, 
-						(struct sockaddr *)&caddr, bits))
+			!comp_with_mask_sock((struct sockaddr *)&addr,
+					     (struct sockaddr *)&caddr, bits))
 			continue;
 
-		dlinkDestroy(ptr, &glines);
+		rb_dlinkDestroy(ptr, &glines);
 		delete_one_address_conf(aconf->host, aconf);
 		return YES;
 	}
 
 	return NO;
 }
+
+/*
+ * expire_pending_glines
+ * 
+ * inputs       - NONE
+ * output       - NONE
+ * side effects -
+ *
+ * Go through the pending gline list, expire any that haven't had
+ * enough "votes" in the time period allowed
+ */
+static void
+expire_pending_glines(void *unused)
+{
+	rb_dlink_node *pending_node;
+	rb_dlink_node *next_node;
+	struct gline_pending *glp_ptr;
+
+	RB_DLINK_FOREACH_SAFE(pending_node, next_node, pending_glines.head)
+	{
+		glp_ptr = pending_node->data;
+
+		if(((glp_ptr->last_gline_time + GLINE_PENDING_EXPIRE) <=
+		    rb_current_time()) || find_is_glined(glp_ptr->host, glp_ptr->user))
+
+		{
+			rb_free(glp_ptr->reason1);
+			rb_free(glp_ptr->reason2);
+			rb_free(glp_ptr);
+			rb_dlinkDestroy(pending_node, &pending_glines);
+		}
+	}
+}
