@@ -29,34 +29,30 @@
  * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  * POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: monitor.c 23975 2007-06-30 22:34:38Z jilles $
+ * $Id: monitor.c 26094 2008-09-19 15:33:46Z androsyn $
  */
 #include "stdinc.h"
-#include "tools.h"
-#include "client.h"
-#include "memory.h"
-#include "balloc.h"
+#include "struct.h"
 #include "monitor.h"
 #include "hash.h"
-#include "event.h"
 #include "numeric.h"
+#include "ircd.h"
+#include "match.h"
+#include "send.h"
 
-static struct monitor *monitorTable[MONITOR_HASH_SIZE];
-BlockHeap *monitor_heap;
-
-static void cleanup_monitor(void *unused);
+struct monitor *monitorTable[MONITOR_HASH_SIZE];
+static rb_bh *monitor_heap;
 
 void
 init_monitor(void)
 {
-	monitor_heap = BlockHeapCreate(sizeof(struct monitor), MONITOR_HEAP_SIZE);
-	eventAddIsh("cleanup_monitor", cleanup_monitor, NULL, 3600);
+	monitor_heap = rb_bh_create(sizeof(struct monitor), MONITOR_HEAP_SIZE, "monitor_heap");
 }
 
 static inline unsigned int
 hash_monitor_nick(const char *name)
 {
-	return fnv_hash_upper((const unsigned char *) name, MONITOR_HASH_BITS);
+	return fnv_hash_upper((const unsigned char *)name, MONITOR_HASH_BITS, 0);
 }
 
 struct monitor *
@@ -74,8 +70,8 @@ find_monitor(const char *name, int add)
 
 	if(add)
 	{
-		monptr = BlockHeapAlloc(monitor_heap);
-		strlcpy(monptr->name, name, sizeof(monptr->name));
+		monptr = rb_bh_alloc(monitor_heap);
+		rb_strlcpy(monptr->name, name, sizeof(monptr->name));
 
 		monptr->hnext = monitorTable[hashv];
 		monitorTable[hashv] = monptr;
@@ -86,6 +82,13 @@ find_monitor(const char *name, int add)
 	return NULL;
 }
 
+void
+free_monitor(struct monitor *monptr)
+{
+	rb_bh_free(monitor_heap, monptr);
+}
+
+
 /* monitor_signon()
  *
  * inputs	- client who has just connected
@@ -103,8 +106,8 @@ monitor_signon(struct Client *client_p)
 	if(monptr == NULL)
 		return;
 
-	ircsnprintf(buf, sizeof(buf), "%s!%s@%s",
-		    client_p->name, client_p->username, client_p->host);
+	rb_snprintf(buf, sizeof(buf), "%s!%s@%s", client_p->name, client_p->username,
+		    client_p->host);
 
 	sendto_monitor(monptr, form_str(RPL_MONONLINE), me.name, "*", buf);
 }
@@ -125,56 +128,27 @@ monitor_signoff(struct Client *client_p)
 	if(monptr == NULL)
 		return;
 
-	sendto_monitor(monptr, form_str(RPL_MONOFFLINE), me.name, "*",
-			client_p->name);
+	sendto_monitor(monptr, form_str(RPL_MONOFFLINE), me.name, "*", client_p->name);
 }
 
+
 void
 clear_monitor(struct Client *client_p)
 {
 	struct monitor *monptr;
-	dlink_node *ptr, *next_ptr;
+	rb_dlink_node *ptr, *next_ptr;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, client_p->localClient->monitor_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, client_p->localClient->monitor_list.head)
 	{
 		monptr = ptr->data;
 
 		/* we leave the actual entry around with no users, itll be
 		 * cleaned up periodically by cleanup_monitor() --anfl
 		 */
-		dlinkFindDestroy(client_p, &monptr->users);
-		free_dlink_node(ptr);
+		rb_dlinkFindDestroy(client_p, &monptr->users);
+		rb_free_rb_dlink_node(ptr);
 	}
 
 	client_p->localClient->monitor_list.head = client_p->localClient->monitor_list.tail = NULL;
 	client_p->localClient->monitor_list.length = 0;
 }
-
-static void
-cleanup_monitor(void *unused)
-{
-	struct monitor *last_ptr = NULL;
-	struct monitor *next_ptr, *ptr;
-	int i;
-
-	for(i = 0; i < MONITOR_HASH_SIZE; i++)
-	{
-		last_ptr = NULL;
-		for(ptr = monitorTable[i]; ptr; ptr = next_ptr)
-		{
-			next_ptr = ptr->hnext;
-
-			if(!dlink_list_length(&ptr->users))
-			{
-				if(last_ptr)
-					last_ptr->hnext = next_ptr;
-				else
-					monitorTable[i] = next_ptr;
-
-				BlockHeapFree(monitor_heap, ptr);
-			}
-			else
-				last_ptr = ptr;
-		}
-	}
-}
