@@ -18,28 +18,25 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: parse.c 21277 2005-11-21 20:10:49Z leeh $
+ *  $Id: parse.c 26330 2008-12-17 08:41:36Z androsyn $
  */
 
 #include "stdinc.h"
+#include "struct.h"
 #include "parse.h"
 #include "client.h"
 #include "channel.h"
-#include "common.h"
 #include "hash.h"
-#include "irc_string.h"
-#include "sprintf_irc.h"
+#include "match.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "s_log.h"
 #include "s_stats.h"
 #include "send.h"
-#include "msg.h"
 #include "s_conf.h"
-#include "memory.h"
 #include "s_serv.h"
 
 /*
@@ -48,16 +45,16 @@
 static char *sender;
 
 /* parv[0] == source, and parv[LAST] == NULL */
-static const char *para[MAXPARA + 2];
+static char *para[MAXPARA + 2];
 
-static void cancel_clients(struct Client *, struct Client *, char *);
+static void cancel_clients(struct Client *, struct Client *);
 static void remove_unknown(struct Client *, char *, char *);
 
-static void do_numeric(char[], struct Client *, struct Client *, int, const char **);
+static void do_numeric(char[], struct Client *, struct Client *, int, char **);
 
-static int handle_command(struct Message *, struct Client *, struct Client *, int, const char**);
+static int handle_command(struct Message *, struct Client *, struct Client *, int, const char **);
 
-static int cmd_hash(const char *p);
+static uint32_t cmd_hash(const char *p);
 static struct Message *hash_parse(const char *);
 
 struct MessageHash *msg_hash_table[MAX_MSG_HASH];
@@ -67,14 +64,14 @@ static char buffer[1024];
 /* turn a string into a parc/parv pair */
 
 
-static inline int
-string_to_array(char *string, const char **parv)
+int
+string_to_array(char *string, char **parv)
 {
 	char *p, *buf = string;
 	int x = 1;
 
 	parv[x] = NULL;
-	while (*buf == ' ')	/* skip leading spaces */
+	while(*buf == ' ')	/* skip leading spaces */
 		buf++;
 	if(*buf == '\0')	/* ignore all-space args */
 		return x;
@@ -100,13 +97,13 @@ string_to_array(char *string, const char **parv)
 			else
 				return x;
 		}
-		while (*buf == ' ')
+		while(*buf == ' ')
 			buf++;
 		if(*buf == '\0')
 			return x;
 	}
 	/* we can go upto parv[MAXPARA], as parv[0] is taken by source */
-	while (x < MAXPARA);
+	while(x < MAXPARA);
 
 	if(*p == ':')
 		p++;
@@ -132,14 +129,16 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
 	struct Message *mptr;
 
 	s_assert(MyConnect(client_p));
-	s_assert(client_p->localClient->fd >= 0);
 	if(IsAnyDead(client_p))
 		return;
 
-	for (ch = pbuffer; *ch == ' '; ch++)	/* skip spaces */
+	for(ch = pbuffer; *ch == ' '; ch++)	/* skip spaces */
 		/* null statement */ ;
 
-	para[0] = from->name;
+	if(from->name != NULL)
+		para[0] = LOCAL_COPY(from->name);
+	else
+		para[0] = NULL;
 
 	if(*ch == ':')
 	{
@@ -162,28 +161,28 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
 			/* didnt find any matching client, issue a kill */
 			if(from == NULL)
 			{
-				ServerStats->is_unpf++;
+				ServerStats.is_unpf++;
 				remove_unknown(client_p, sender, pbuffer);
 				return;
 			}
 
-			para[0] = from->name;
+			para[0] = LOCAL_COPY(from->name);
 
 			/* fake direction, hmm. */
 			if(from->from != client_p)
 			{
-				ServerStats->is_wrdi++;
-				cancel_clients(client_p, from, pbuffer);
+				ServerStats.is_wrdi++;
+				cancel_clients(client_p, from);
 				return;
 			}
 		}
-		while (*ch == ' ')
+		while(*ch == ' ')
 			ch++;
 	}
 
 	if(*ch == '\0')
 	{
-		ServerStats->is_empt++;
+		ServerStats.is_empt++;
 		return;
 	}
 
@@ -204,7 +203,7 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
 	{
 		mptr = NULL;
 		numeric = ch;
-		ServerStats->is_num++;
+		ServerStats.is_num++;
 		s = ch + 3;	/* I know this is ' ' from above if */
 		*s++ = '\0';	/* blow away the ' ', and point s to next part */
 	}
@@ -233,11 +232,11 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
 			 */
 			if(pbuffer[0] != '\0')
 			{
-				if(IsPerson(from))
+				if(IsClient(from))
 					sendto_one(from, form_str(ERR_UNKNOWNCOMMAND),
 						   me.name, from->name, ch);
 			}
-			ServerStats->is_unco++;
+			ServerStats.is_unco++;
 			return;
 		}
 
@@ -262,18 +261,19 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
 		return;
 	}
 
-	if(handle_command(mptr, client_p, from, i, /* XXX discards const!!! */ (const char **)para) < -1)
+	if(handle_command(mptr, client_p, from, i,	/* XXX discards const!!! */
+			  (const char **)para) < -1)
 	{
 		char *p;
-		for (p = pbuffer; p <= end; p += 8)
+		for(p = pbuffer; p <= end; p += 8)
 		{
 			/* HACK HACK */
 			/* Its expected this nasty code can be removed
 			 * or rewritten later if still needed.
 			 */
-			if((unsigned long) (p + 8) > (unsigned long) end)
+			if((unsigned long)(p + 8) > (unsigned long)end)
 			{
-				for (; p <= end; p++)
+				for(; p <= end; p++)
 				{
 					ilog(L_MAIN, "%02x |%c", p[0], p[0]);
 				}
@@ -301,10 +301,11 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
  */
 static int
 handle_command(struct Message *mptr, struct Client *client_p,
-	       struct Client *from, int i, const char** hpara)
+	       struct Client *from, int i, const char **hpara)
 {
 	struct MessageEntry ehandler;
 	MessageHandler handler = 0;
+	static time_t last_warning;
 
 	if(IsAnyDead(client_p))
 		return -1;
@@ -331,15 +332,14 @@ handle_command(struct Message *mptr, struct Client *client_p,
 	handler = ehandler.handler;
 
 	/* check right amount of params is passed... --is */
-	if(i < ehandler.min_para || 
+	if(i < ehandler.min_para ||
 	   (ehandler.min_para && EmptyString(hpara[ehandler.min_para - 1])))
 	{
 		if(!IsServer(client_p))
 		{
 			sendto_one(client_p, form_str(ERR_NEEDMOREPARAMS),
-				   me.name, 
-				   EmptyString(client_p->name) ? "*" : client_p->name, 
-				   mptr->cmd);
+				   me.name,
+				   EmptyString(client_p->name) ? "*" : client_p->name, mptr->cmd);
 			if(MyClient(client_p))
 				return (1);
 			else
@@ -360,6 +360,18 @@ handle_command(struct Message *mptr, struct Client *client_p,
 	}
 
 	(*handler) (client_p, from, i, hpara);
+	if(!IsAnyDead(client_p) && IsCork(client_p) && !IsCapable(client_p, CAP_ZIP))
+	{
+		if(last_warning + 300 <= rb_current_time())
+		{
+			sendto_realops_flags(UMODE_DEBUG, L_ALL,
+					     "Bug: client %s was left corked after command %s",
+					     client_p->name, mptr->cmd);
+			last_warning = rb_current_time();
+		}
+		client_p->localClient->cork_count = 0;
+		send_pop_queue(client_p);
+	}
 	return (1);
 }
 
@@ -381,7 +393,7 @@ handle_encap(struct Client *client_p, struct Client *source_p,
 	ehandler = mptr->handlers[ENCAP_HANDLER];
 	handler = ehandler.handler;
 
-	if(parc < ehandler.min_para || 
+	if(parc < ehandler.min_para ||
 	   (ehandler.min_para && EmptyString(parv[ehandler.min_para - 1])))
 		return;
 
@@ -426,17 +438,17 @@ mod_add_cmd(struct Message *msg)
 
 	msgindex = cmd_hash(msg->cmd);
 
-	for (ptr = msg_hash_table[msgindex]; ptr; ptr = ptr->next)
+	for(ptr = msg_hash_table[msgindex]; ptr; ptr = ptr->next)
 	{
 		if(strcasecmp(msg->cmd, ptr->cmd) == 0)
 			return;	/* Its already added */
 		last_ptr = ptr;
 	}
 
-	new_ptr = (struct MessageHash *) MyMalloc(sizeof(struct MessageHash));
+	new_ptr = rb_malloc(sizeof(struct MessageHash));
 
 	new_ptr->next = NULL;
-	DupString(new_ptr->cmd, msg->cmd);
+	new_ptr->cmd = rb_strdup(msg->cmd);
 	new_ptr->msg = msg;
 
 	msg->count = 0;
@@ -468,16 +480,16 @@ mod_del_cmd(struct Message *msg)
 
 	msgindex = cmd_hash(msg->cmd);
 
-	for (ptr = msg_hash_table[msgindex]; ptr; ptr = ptr->next)
+	for(ptr = msg_hash_table[msgindex]; ptr; ptr = ptr->next)
 	{
 		if(strcasecmp(msg->cmd, ptr->cmd) == 0)
 		{
-			MyFree(ptr->cmd);
+			rb_free(ptr->cmd);
 			if(last_ptr != NULL)
 				last_ptr->next = ptr->next;
 			else
 				msg_hash_table[msgindex] = ptr->next;
-			MyFree(ptr);
+			rb_free(ptr);
 			return;
 		}
 		last_ptr = ptr;
@@ -498,7 +510,7 @@ hash_parse(const char *cmd)
 
 	msgindex = cmd_hash(cmd);
 
-	for (ptr = msg_hash_table[msgindex]; ptr; ptr = ptr->next)
+	for(ptr = msg_hash_table[msgindex]; ptr; ptr = ptr->next)
 	{
 		if(strcasecmp(cmd, ptr->cmd) == 0)
 			return (ptr->msg);
@@ -514,49 +526,22 @@ hash_parse(const char *cmd)
  * output	- hash index
  * side effects - NONE
  *
- * BUGS		- This a HORRIBLE hash function
+ * BUGS		- This was a horrible hash function, now its an okay one...
  */
-static int
+static uint32_t
 cmd_hash(const char *p)
 {
-	int hash_val = 0;
+	uint32_t hash_val = 0, q = 1, n;
 
-	while (*p)
+	while(*p)
 	{
-		hash_val += ((int) (*p) & 0xDF);
-		p++;
+		n = ToUpper(*p++);
+		hash_val += ((n) + (q++ << 1)) ^ ((n) << 2);
 	}
-
-	return (hash_val % MAX_MSG_HASH);
+	/* note that 9 comes from 2^9 = MAX_MSG_HASH */
+	return (hash_val >> (32 - 9)) ^ (hash_val & (MAX_MSG_HASH - 1));
 }
 
-/*
- * report_messages
- *
- * inputs	- pointer to client to report to
- * output	- NONE
- * side effects	- NONE
- */
-void
-report_messages(struct Client *source_p)
-{
-	int i;
-	struct MessageHash *ptr;
-
-	for (i = 0; i < MAX_MSG_HASH; i++)
-	{
-		for (ptr = msg_hash_table[i]; ptr; ptr = ptr->next)
-		{
-			s_assert(ptr->msg != NULL);
-			s_assert(ptr->cmd != NULL);
-
-			sendto_one_numeric(source_p, RPL_STATSCOMMANDS, 
-					   form_str(RPL_STATSCOMMANDS),
-					   ptr->cmd, ptr->msg->count, 
-					   ptr->msg->bytes, ptr->msg->rcount);
-		}
-	}
-}
 
 /* cancel_clients()
  *
@@ -566,7 +551,7 @@ report_messages(struct Client *source_p)
  * side effects -
  */
 static void
-cancel_clients(struct Client *client_p, struct Client *source_p, char *cmd)
+cancel_clients(struct Client *client_p, struct Client *source_p)
 {
 	/* ok, fake prefix happens naturally during a burst on a nick
 	 * collision with TS5, we cant kill them because one client has to
@@ -576,16 +561,15 @@ cancel_clients(struct Client *client_p, struct Client *source_p, char *cmd)
 	{
 		sendto_realops_flags(UMODE_DEBUG, L_ALL,
 				     "Message for %s[%s] from %s",
-				     source_p->name, source_p->from->name,
-				     client_p->name);
+				     source_p->name, source_p->from->name, client_p->name);
 	}
 	else
 	{
 		sendto_realops_flags(UMODE_DEBUG, L_ALL,
 				     "Message for %s[%s@%s!%s] from %s (TS, ignored)",
-				     source_p->name, source_p->username,
-				     source_p->host, source_p->from->name,
-				     client_p->name);
+				     source_p->name,
+				     source_p->username,
+				     source_p->host, source_p->from->name, client_p->name);
 	}
 }
 
@@ -600,25 +584,22 @@ remove_unknown(struct Client *client_p, char *lsender, char *lbuffer)
 {
 	int slen = strlen(lsender);
 
-	/* meepfoo	is a nickname (KILL)
-	 * #XXXXXXXX	is a UID (KILL)
-	 * #XX		is a SID (SQUIT)
-	 * meep.foo	is a server (SQUIT)
+	/* meepfoo      is a nickname (KILL)
+	 * #XXXXXXXX    is a UID (KILL)
+	 * #XX          is a SID (SQUIT)
+	 * meep.foo     is a server (SQUIT)
 	 */
-	if((IsDigit(lsender[0]) && slen == 3) || 
-	   (strchr(lsender, '.') != NULL))
+	if((IsDigit(lsender[0]) && slen == 3) || (strchr(lsender, '.') != NULL))
 	{
 		sendto_realops_flags(UMODE_DEBUG, L_ALL,
 				     "Unknown prefix (%s) from %s, Squitting %s",
 				     lbuffer, client_p->name, lsender);
 
-		sendto_one(client_p,
-			   ":%s SQUIT %s :(Unknown prefix (%s) from %s)",
-			   get_id(&me, client_p), lsender, 
-			   lbuffer, client_p->name);
+		sendto_one(client_p, ":%s SQUIT %s :(Unknown prefix (%s) from %s)",
+			   get_id(&me, client_p), lsender, lbuffer, client_p->name);
 	}
 	else
-		sendto_one(client_p, ":%s KILL %s :%s (Unknown Client)", 
+		sendto_one(client_p, ":%s KILL %s :%s (Unknown Client)",
 			   get_id(&me, client_p), lsender, me.name);
 }
 
@@ -639,7 +620,7 @@ remove_unknown(struct Client *client_p, char *lsender, char *lbuffer)
  *      a ping pong error message...
  */
 static void
-do_numeric(char numeric[], struct Client *client_p, struct Client *source_p, int parc, const char *parv[])
+do_numeric(char numeric[], struct Client *client_p, struct Client *source_p, int parc, char *parv[])
 {
 	struct Client *target_p;
 	struct Channel *chptr;
@@ -663,12 +644,12 @@ do_numeric(char numeric[], struct Client *client_p, struct Client *source_p, int
 		char *t = buffer;	/* Current position within the buffer */
 		int i;
 		int tl;		/* current length of presently being built string in t */
-		for (i = 2; i < (parc - 1); i++)
+		for(i = 2; i < (parc - 1); i++)
 		{
-			tl = ircsprintf(t, " %s", parv[i]);
+			tl = rb_sprintf(t, " %s", parv[i]);
 			t += tl;
 		}
-		ircsprintf(t, " :%s", parv[parc - 1]);
+		rb_sprintf(t, " :%s", parv[parc - 1]);
 	}
 
 	if((target_p = find_client(parv[1])) != NULL)
@@ -696,8 +677,7 @@ do_numeric(char numeric[], struct Client *client_p, struct Client *source_p, int
 			/* note, now we send PING on server connect, we can
 			 * also get ERR_NOSUCHSERVER..
 			 */
-			if(atoi(numeric) != ERR_NOSUCHNICK &&
-			   atoi(numeric) != ERR_NOSUCHSERVER)
+			if(atoi(numeric) != ERR_NOSUCHNICK && atoi(numeric) != ERR_NOSUCHSERVER)
 				sendto_realops_flags(UMODE_ALL, L_ADMIN,
 						     "*** %s(via %s) sent a %s numeric to me: %s",
 						     source_p->name,
@@ -717,9 +697,8 @@ do_numeric(char numeric[], struct Client *client_p, struct Client *source_p, int
 			return;
 
 		/* Fake it for server hiding, if its our client */
-		sendto_one(target_p, ":%s %s %s%s", 
-			   get_id(source_p, target_p), numeric, 
-			   get_id(target_p, target_p), buffer);
+		sendto_one(target_p, ":%s %s %s%s",
+			   get_id(source_p, target_p), numeric, get_id(target_p, target_p), buffer);
 		return;
 	}
 	else if((chptr = find_channel(parv[1])) != NULL)
