@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
  *  Copyright (C) 1996-2002 Hybrid Development Team
- *  Copyright (C) 2002-2005 ircd-ratbox development team
+ *  Copyright (C) 2002-2004 ircd-ratbox development team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,37 +18,35 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  *  USA
  *
- *  $Id: send.c 23975 2007-06-30 22:34:38Z jilles $
+ *  $Id: send.c 26094 2008-09-19 15:33:46Z androsyn $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "struct.h"
 #include "send.h"
 #include "channel.h"
 #include "class.h"
 #include "client.h"
 #include "common.h"
-#include "irc_string.h"
+#include "match.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "commio.h"
 #include "s_serv.h"
-#include "sprintf_irc.h"
 #include "s_conf.h"
 #include "s_newconf.h"
-#include "linebuf.h"
 #include "s_log.h"
-#include "memory.h"
 #include "hook.h"
 #include "monitor.h"
 
-/* send the message to the link the target is attached to */
-#define send_linebuf(a,b) _send_linebuf((a->from ? a->from : a) ,b)
+#define LOG_BUFSIZE 2048
+
+uint32_t current_serial = 0L;
+static void send_queued_write(rb_fde_t *F, void *data);
+static void send_queued(struct Client *to);
 
-unsigned long current_serial = 0L;
 
 /* send_linebuf()
  *
@@ -57,7 +55,7 @@ unsigned long current_serial = 0L;
  * side effects - linebuf is attached to client
  */
 static int
-_send_linebuf(struct Client *to, buf_head_t *linebuf)
+send_linebuf(struct Client *to, buf_head_t * linebuf)
 {
 	if(IsMe(to))
 	{
@@ -68,20 +66,19 @@ _send_linebuf(struct Client *to, buf_head_t *linebuf)
 	if(!MyConnect(to) || IsIOError(to))
 		return 0;
 
-	if(linebuf_len(&to->localClient->buf_sendq) > get_sendq(to))
+	if(rb_linebuf_len(&to->localClient->buf_sendq) > get_sendq(to))
 	{
 		if(IsServer(to))
 		{
 			sendto_realops_flags(UMODE_ALL, L_ALL,
 					     "Max SendQ limit exceeded for %s: %u > %lu",
 					     to->name,
-					     linebuf_len(&to->localClient->buf_sendq), 
+					     rb_linebuf_len(&to->localClient->buf_sendq),
 					     get_sendq(to));
 
 			ilog(L_SERVER, "Max SendQ limit exceeded for %s: %u > %lu",
 			     log_client_name(to, SHOW_IP),
-			     linebuf_len(&to->localClient->buf_sendq), 
-			     get_sendq(to));
+			     rb_linebuf_len(&to->localClient->buf_sendq), get_sendq(to));
 		}
 
 		dead_link(to, 1);
@@ -92,7 +89,7 @@ _send_linebuf(struct Client *to, buf_head_t *linebuf)
 		/* just attach the linebuf to the sendq instead of
 		 * generating a new one
 		 */
-		linebuf_attach(&to->localClient->buf_sendq, linebuf);
+		rb_linebuf_attach(&to->localClient->buf_sendq, linebuf);
 	}
 
 	/*
@@ -102,25 +99,37 @@ _send_linebuf(struct Client *to, buf_head_t *linebuf)
 	 */
 	to->localClient->sendM += 1;
 	me.localClient->sendM += 1;
-	if(linebuf_len(&to->localClient->buf_sendq) > 0)
-		send_queued_write(to->localClient->fd, to);
+
+	if(rb_linebuf_len(&to->localClient->buf_sendq) > 0)
+		send_queued(to);
 	return 0;
 }
 
-/* send_linebuf_remote()
+void
+send_pop_queue(struct Client *to)
+{
+	if(to->from != NULL)
+		to = to->from;
+	if(!MyConnect(to) || IsIOError(to))
+		return;
+	if(rb_linebuf_len(&to->localClient->buf_sendq) > 0)
+		send_queued(to);
+}
+
+/* send_rb_linebuf_remote()
  *
  * inputs	- client to attach to, sender, linebuf
  * outputs	-
  * side effects - client has linebuf attached
  */
 static void
-send_linebuf_remote(struct Client *to, struct Client *from, buf_head_t *linebuf)
+send_rb_linebuf_remote(struct Client *to, struct Client *from, buf_head_t * linebuf)
 {
 	if(to->from)
 		to = to->from;
 
 	/* test for fake direction */
-	if(!MyClient(from) && IsPerson(to) && (to == from->from))
+	if(!MyClient(from) && IsClient(to) && (to == from->from))
 	{
 		if(IsServer(from))
 		{
@@ -135,8 +144,7 @@ send_linebuf_remote(struct Client *to, struct Client *from, buf_head_t *linebuf)
 				     to->name, to->username, to->host,
 				     from->name, from->username, from->host, to->from->name);
 		kill_client_serv_butone(NULL, to, "%s (%s[%s@%s] Ghosted %s)",
-					me.name, to->name, to->username,
-					to->host, to->from->name);
+					me.name, to->name, to->username, to->host, to->from->name);
 
 		to->flags |= FLAGS_KILLED;
 
@@ -144,22 +152,14 @@ send_linebuf_remote(struct Client *to, struct Client *from, buf_head_t *linebuf)
 		return;
 	}
 
-	_send_linebuf(to, linebuf);
+	send_linebuf(to, linebuf);
 	return;
 }
 
-/* send_queued_write()
- *
- * inputs	- fd to have queue sent, client we're sending to
- * outputs	- contents of queue
- * side effects - write is rescheduled if queue isnt emptied
- */
-void
-send_queued_write(int fd, void *data)
+static void
+send_queued(struct Client *to)
 {
-	struct Client *to = data;
 	int retlen;
-	int flags;
 #ifdef USE_IODEBUG_HOOKS
 	hook_data_int hd;
 #endif
@@ -167,122 +167,93 @@ send_queued_write(int fd, void *data)
 	if(IsIOError(to))
 		return;
 
+	/* Something wants us to not send anything currently */
+	if(IsCork(to))
+		return;
+
+	/* try to flush later when the write event resets this */
+	if(IsFlush(to))
+		return;
+
 #ifdef USE_IODEBUG_HOOKS
 	hd.client = to;
 	if(to->localClient->buf_sendq.list.head)
 		hd.arg1 = ((buf_line_t *) to->localClient->buf_sendq.list.head->data)->buf +
-	                     to->localClient->buf_sendq.writeofs;
+			to->localClient->buf_sendq.writeofs;
 #endif
 
-	if(linebuf_len(&to->localClient->buf_sendq))
+	if(rb_linebuf_len(&to->localClient->buf_sendq))
 	{
-		while ((retlen =
-			linebuf_flush(to->localClient->fd, &to->localClient->buf_sendq)) > 0)
+		while((retlen =
+		       rb_linebuf_flush(to->localClient->F, &to->localClient->buf_sendq)) > 0)
 		{
 			/* We have some data written .. update counters */
 #ifdef USE_IODEBUG_HOOKS
-                        hd.arg2 = retlen;
-                        call_hook(h_iosend_id, &hd);
+			hd.arg2 = retlen;
+			call_hook(h_iosend_id, &hd);
 
-                        if(to->localClient->buf_sendq.list.head)
-                                hd.arg1 =
-                                        ((buf_line_t *) to->localClient->buf_sendq.list.head->
-                                         data)->buf + to->localClient->buf_sendq.writeofs;
+			if(to->localClient->buf_sendq.list.head)
+				hd.arg1 =
+					((buf_line_t *) to->localClient->buf_sendq.list.
+					 head->data)->buf + to->localClient->buf_sendq.writeofs;
 #endif
-     
+
+			ClearFlush(to);
 
 			to->localClient->sendB += retlen;
 			me.localClient->sendB += retlen;
-			if(to->localClient->sendB > 1023)
-			{
-				to->localClient->sendK += (to->localClient->sendB >> 10);
-				to->localClient->sendB &= 0x03ff;	/* 2^10 = 1024, 3ff = 1023 */
-			}
-			else if(me.localClient->sendB > 1023)
-			{
-				me.localClient->sendK += (me.localClient->sendB >> 10);
-				me.localClient->sendB &= 0x03ff;
-			}
 		}
 
-		if(retlen == 0 || (retlen < 0 && !ignoreErrno(errno)))
+		if(retlen == 0 || (retlen < 0 && !rb_ignore_errno(errno)))
 		{
 			dead_link(to, 0);
 			return;
 		}
 	}
-	if(ignoreErrno(errno))
-		flags = COMM_SELECT_WRITE|COMM_SELECT_RETRY;
+	if(rb_linebuf_len(&to->localClient->buf_sendq))
+	{
+		SetFlush(to);
+		rb_setselect(to->localClient->F, RB_SELECT_WRITE, send_queued_write, to);
+	}
 	else
-		flags = COMM_SELECT_WRITE;
-	if(linebuf_len(&to->localClient->buf_sendq))
-	comm_setselect(fd, FDLIST_IDLECLIENT, flags,
-			       send_queued_write, to);
+		ClearFlush(to);
+
 }
 
-/* send_queued_slink_write()
+/* send_queued_write()
  *
  * inputs	- fd to have queue sent, client we're sending to
  * outputs	- contents of queue
- * side effects - write is rescheduled if queue isnt emptied
+ * side effects - write is scheduled if queue isnt emptied
  */
-void
-send_queued_slink_write(int fd, void *data)
+static void
+send_queued_write(rb_fde_t *F, void *data)
 {
 	struct Client *to = data;
-	int retlen;
+	ClearFlush(to);
+	send_queued(to);
+}
 
-	/*
-	 ** Once socket is marked dead, we cannot start writing to it,
-	 ** even if the error is removed...
-	 */
-	if(IsIOError(to))
+/* sendto_one_buffer()
+ *
+ * inputs	- client to send to, buffer
+ * outputs	- client has message put into its queue
+ */
+void
+sendto_one_buffer(struct Client *target_p, const char *buffer)
+{
+	buf_head_t linebuf;
+
+	if(target_p->from != NULL)
+		target_p = target_p->from;
+
+	if(IsIOError(target_p))
 		return;
 
-	/* Next, lets try to write some data */
-	if(to->localClient->slink->slinkq)
-	{
-		retlen = write(to->localClient->slink->ctrlfd,
-			      to->localClient->slink->slinkq + to->localClient->slink->slinkq_ofs,
-			      to->localClient->slink->slinkq_len);
-
-		if(retlen < 0)
-		{
-			/* If we have a fatal error */
-			if(!ignoreErrno(errno))
-			{
-				dead_link(to, 0);
-				return;
-			}
-		}
-		/* 0 bytes is an EOF .. */
-		else if(retlen == 0)
-		{
-			dead_link(to, 0);
-			return;
-		}
-		else
-		{
-			to->localClient->slink->slinkq_len -= retlen;
-
-			s_assert(to->localClient->slink->slinkq_len >= 0);
-			if(to->localClient->slink->slinkq_len)
-				to->localClient->slink->slinkq_ofs += retlen;
-			else
-			{
-				to->localClient->slink->slinkq_ofs = 0;
-				MyFree(to->localClient->slink->slinkq);
-				to->localClient->slink->slinkq = NULL;
-			}
-		}
-	}
-
-	/* if we have any more data, reschedule a write */
-	if(to->localClient->slink->slinkq_len)
-		comm_setselect(to->localClient->slink->ctrlfd, 
-				FDLIST_IDLECLIENT,
-				COMM_SELECT_WRITE|COMM_SELECT_RETRY, 
-				send_queued_slink_write, to);
+	rb_linebuf_newbuf(&linebuf);
+	rb_linebuf_putbuf(&linebuf, buffer);
+	send_linebuf(target_p, &linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 /* sendto_one()
@@ -304,18 +275,19 @@ sendto_one(struct Client *target_p, const char *pattern, ...)
 	if(IsIOError(target_p))
 		return;
 
-	linebuf_newbuf(&linebuf);
+	rb_linebuf_newbuf(&linebuf);
 
 	va_start(args, pattern);
-	linebuf_putmsg(&linebuf, pattern, &args, NULL);
+	rb_linebuf_putmsg(&linebuf, pattern, &args, NULL);
 	va_end(args);
 
-	_send_linebuf(target_p, &linebuf);
+	send_linebuf(target_p, &linebuf);
 
-	linebuf_donebuf(&linebuf);
+	rb_linebuf_donebuf(&linebuf);
 
 }
 
+
 /* sendto_one_prefix()
  *
  * inputs	- client to send to, va_args
@@ -345,16 +317,31 @@ sendto_one_prefix(struct Client *target_p, struct Client *source_p,
 		return;
 	}
 
-	linebuf_newbuf(&linebuf);
+	rb_linebuf_newbuf(&linebuf);
 	va_start(args, pattern);
-	linebuf_putmsg(&linebuf, pattern, &args,
-		       ":%s %s %s ",
-		       get_id(source_p, target_p),
-		       command, get_id(target_p, target_p));
+	rb_linebuf_putmsg(&linebuf, pattern, &args,
+			  ":%s %s %s ",
+			  get_id(source_p, target_p), command, get_id(target_p, target_p));
 	va_end(args);
 
-	_send_linebuf(dest_p, &linebuf);
-	linebuf_donebuf(&linebuf);
+	send_linebuf(dest_p, &linebuf);
+	rb_linebuf_donebuf(&linebuf);
+}
+
+/*
+ * sendto_one_notice_local()
+ * inputs	- client to send to, va_args
+ * outputs	- client has a NOTICE put into its queue
+ * side effects - fast path for local clients 
+ */
+static void
+sendto_one_notice_local(struct Client *target_p, const char *pattern, va_list * ap)
+{
+	buf_head_t linebuf;
+	rb_linebuf_newbuf(&linebuf);
+	rb_linebuf_putmsg(&linebuf, pattern, ap, ":%s NOTICE %s ", me.name, target_p->name);
+	send_linebuf(target_p, &linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 /* sendto_one_notice()
@@ -370,11 +357,17 @@ sendto_one_notice(struct Client *target_p, const char *pattern, ...)
 	va_list args;
 	buf_head_t linebuf;
 
-	/* send remote if to->from non NULL */
-	if(target_p->from != NULL)
-		dest_p = target_p->from;
-	else
-		dest_p = target_p;
+	if(MyConnect(target_p))
+	{
+		if(IsIOError(target_p))
+			return;
+		va_start(args, pattern);
+		sendto_one_notice_local(target_p, pattern, &args);
+		va_end(args);
+		return;
+	}
+
+	dest_p = target_p->from;
 
 	if(IsIOError(dest_p))
 		return;
@@ -385,15 +378,14 @@ sendto_one_notice(struct Client *target_p, const char *pattern, ...)
 		return;
 	}
 
-	linebuf_newbuf(&linebuf);
+	rb_linebuf_newbuf(&linebuf);
 	va_start(args, pattern);
-	linebuf_putmsg(&linebuf, pattern, &args,
-		       ":%s NOTICE %s ",
-		       get_id(&me, target_p), get_id(target_p, target_p));
+	rb_linebuf_putmsg(&linebuf, pattern, &args,
+			  ":%s NOTICE %s ", get_id(&me, target_p), get_id(target_p, target_p));
 	va_end(args);
 
-	_send_linebuf(dest_p, &linebuf);
-	linebuf_donebuf(&linebuf);
+	send_linebuf(dest_p, &linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 
@@ -425,16 +417,15 @@ sendto_one_numeric(struct Client *target_p, int numeric, const char *pattern, ..
 		return;
 	}
 
-	linebuf_newbuf(&linebuf);
+	rb_linebuf_newbuf(&linebuf);
 	va_start(args, pattern);
-	linebuf_putmsg(&linebuf, pattern, &args,
-		       ":%s %03d %s ",
-		       get_id(&me, target_p),
-		       numeric, get_id(target_p, target_p));
+	rb_linebuf_putmsg(&linebuf, pattern, &args,
+			  ":%s %03d %s ",
+			  get_id(&me, target_p), numeric, get_id(target_p, target_p));
 	va_end(args);
 
-	_send_linebuf(dest_p, &linebuf);
-	linebuf_donebuf(&linebuf);
+	send_linebuf(dest_p, &linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 /*
@@ -460,23 +451,23 @@ sendto_server(struct Client *one, struct Channel *chptr, unsigned long caps,
 {
 	va_list args;
 	struct Client *target_p;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 	buf_head_t linebuf;
 
 	/* noone to send to.. */
-	if(dlink_list_length(&serv_list) == 0)
+	if(rb_dlink_list_length(&serv_list) == 0)
 		return;
 
 	if(chptr != NULL && *chptr->chname != '#')
-			return;
+		return;
 
-	linebuf_newbuf(&linebuf);
+	rb_linebuf_newbuf(&linebuf);
 	va_start(args, format);
-	linebuf_putmsg(&linebuf, format, &args, NULL);
+	rb_linebuf_putmsg(&linebuf, format, &args, NULL);
 	va_end(args);
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, serv_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, serv_list.head)
 	{
 		target_p = ptr->data;
 
@@ -492,10 +483,10 @@ sendto_server(struct Client *one, struct Channel *chptr, unsigned long caps,
 		if(!NotCapable(target_p, nocaps))
 			continue;
 
-		_send_linebuf(target_p, &linebuf);
+		send_linebuf(target_p, &linebuf);
 	}
 
-	linebuf_donebuf(&linebuf);
+	rb_linebuf_donebuf(&linebuf);
 
 }
 
@@ -511,37 +502,35 @@ sendto_channel_flags(struct Client *one, int type, struct Client *source_p,
 {
 	static char buf[BUFSIZE];
 	va_list args;
-	buf_head_t linebuf_local;
-	buf_head_t linebuf_name;
-	buf_head_t linebuf_id;
+	buf_head_t rb_linebuf_local;
+	buf_head_t rb_linebuf_name;
+	buf_head_t rb_linebuf_id;
 	struct Client *target_p;
 	struct membership *msptr;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 
-	linebuf_newbuf(&linebuf_local);
-	linebuf_newbuf(&linebuf_name);
-	linebuf_newbuf(&linebuf_id);
+	rb_linebuf_newbuf(&rb_linebuf_local);
+	rb_linebuf_newbuf(&rb_linebuf_name);
+	rb_linebuf_newbuf(&rb_linebuf_id);
 
 	current_serial++;
 
 	va_start(args, pattern);
-	ircvsnprintf(buf, sizeof(buf), pattern, args);
+	rb_vsnprintf(buf, sizeof(buf), pattern, args);
 	va_end(args);
 
 	if(IsServer(source_p))
-		linebuf_putmsg(&linebuf_local, NULL, NULL,
-			       ":%s %s", source_p->name, buf);
+		rb_linebuf_putmsg(&rb_linebuf_local, NULL, NULL, ":%s %s", source_p->name, buf);
 	else
-		linebuf_putmsg(&linebuf_local, NULL, NULL,
-			       ":%s!%s@%s %s",
-			       source_p->name, source_p->username, 
-			       source_p->host, buf);
+		rb_linebuf_putmsg(&rb_linebuf_local, NULL, NULL,
+				  ":%s!%s@%s %s",
+				  source_p->name, source_p->username, source_p->host, buf);
 
-	linebuf_putmsg(&linebuf_name, NULL, NULL, ":%s %s", source_p->name, buf);
-	linebuf_putmsg(&linebuf_id, NULL, NULL, ":%s %s", use_id(source_p), buf);
+	rb_linebuf_putmsg(&rb_linebuf_name, NULL, NULL, ":%s %s", source_p->name, buf);
+	rb_linebuf_putmsg(&rb_linebuf_id, NULL, NULL, ":%s %s", use_id(source_p), buf);
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, chptr->members.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, chptr->members.head)
 	{
 		msptr = ptr->data;
 		target_p = msptr->client_p;
@@ -566,20 +555,21 @@ sendto_channel_flags(struct Client *one, int type, struct Client *source_p,
 			if(target_p->from->localClient->serial != current_serial)
 			{
 				if(has_id(target_p->from))
-					send_linebuf_remote(target_p, source_p, &linebuf_id);
+					send_rb_linebuf_remote(target_p, source_p, &rb_linebuf_id);
 				else
-					send_linebuf_remote(target_p, source_p, &linebuf_name);
+					send_rb_linebuf_remote(target_p, source_p,
+							       &rb_linebuf_name);
 
 				target_p->from->localClient->serial = current_serial;
 			}
 		}
 		else
-			_send_linebuf(target_p, &linebuf_local);
+			send_linebuf(target_p, &rb_linebuf_local);
 	}
 
-	linebuf_donebuf(&linebuf_local);
-	linebuf_donebuf(&linebuf_name);
-	linebuf_donebuf(&linebuf_id);
+	rb_linebuf_donebuf(&rb_linebuf_local);
+	rb_linebuf_donebuf(&rb_linebuf_name);
+	rb_linebuf_donebuf(&rb_linebuf_id);
 }
 
 
@@ -596,16 +586,16 @@ sendto_channel_local(int type, struct Channel *chptr, const char *pattern, ...)
 	buf_head_t linebuf;
 	struct membership *msptr;
 	struct Client *target_p;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
-	
-	linebuf_newbuf(&linebuf); 
-	
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
+
+	rb_linebuf_newbuf(&linebuf);
+
 	va_start(args, pattern);
-	linebuf_putmsg(&linebuf, pattern, &args, NULL);
+	rb_linebuf_putmsg(&linebuf, pattern, &args, NULL);
 	va_end(args);
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, chptr->locmembers.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, chptr->locmembers.head)
 	{
 		msptr = ptr->data;
 		target_p = msptr->client_p;
@@ -616,10 +606,10 @@ sendto_channel_local(int type, struct Channel *chptr, const char *pattern, ...)
 		if(type && ((msptr->flags & type) == 0))
 			continue;
 
-		_send_linebuf(target_p, &linebuf);
+		send_linebuf(target_p, &linebuf);
 	}
 
-	linebuf_donebuf(&linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 /*
@@ -636,39 +626,38 @@ void
 sendto_common_channels_local(struct Client *user, const char *pattern, ...)
 {
 	va_list args;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
-	dlink_node *uptr;
-	dlink_node *next_uptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
+	rb_dlink_node *uptr;
+	rb_dlink_node *next_uptr;
 	struct Channel *chptr;
 	struct Client *target_p;
 	struct membership *msptr;
 	struct membership *mscptr;
 	buf_head_t linebuf;
 
-	linebuf_newbuf(&linebuf);
+	rb_linebuf_newbuf(&linebuf);
 	va_start(args, pattern);
-	linebuf_putmsg(&linebuf, pattern, &args, NULL);
+	rb_linebuf_putmsg(&linebuf, pattern, &args, NULL);
 	va_end(args);
 
 	++current_serial;
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, user->user->channel.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, user->user->channel.head)
 	{
 		mscptr = ptr->data;
 		chptr = mscptr->chptr;
 
-		DLINK_FOREACH_SAFE(uptr, next_uptr, chptr->locmembers.head)
+		RB_DLINK_FOREACH_SAFE(uptr, next_uptr, chptr->locmembers.head)
 		{
 			msptr = uptr->data;
 			target_p = msptr->client_p;
 
-			if(IsIOError(target_p) ||
-			   target_p->localClient->serial == current_serial)
+			if(IsIOError(target_p) || target_p->localClient->serial == current_serial)
 				continue;
 
 			target_p->localClient->serial = current_serial;
-			send_linebuf(target_p, &linebuf);
+			send_linebuf(target_p->from ? target_p->from : target_p, &linebuf);
 		}
 	}
 
@@ -676,9 +665,9 @@ sendto_common_channels_local(struct Client *user, const char *pattern, ...)
 	 * need to send them the data, ie a nick change
 	 */
 	if(MyConnect(user) && (user->localClient->serial != current_serial))
-		send_linebuf(user, &linebuf);
+		send_linebuf(user->from ? user->from : user, &linebuf);
 
-	linebuf_donebuf(&linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 /* sendto_match_butone()
@@ -694,53 +683,51 @@ sendto_match_butone(struct Client *one, struct Client *source_p,
 	static char buf[BUFSIZE];
 	va_list args;
 	struct Client *target_p;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
-	buf_head_t linebuf_local;
-	buf_head_t linebuf_name;
-	buf_head_t linebuf_id;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
+	buf_head_t rb_linebuf_local;
+	buf_head_t rb_linebuf_name;
+	buf_head_t rb_linebuf_id;
 
-	linebuf_newbuf(&linebuf_local);
-	linebuf_newbuf(&linebuf_name);
-	linebuf_newbuf(&linebuf_id);
+	rb_linebuf_newbuf(&rb_linebuf_local);
+	rb_linebuf_newbuf(&rb_linebuf_name);
+	rb_linebuf_newbuf(&rb_linebuf_id);
 
 	va_start(args, pattern);
-	ircvsnprintf(buf, sizeof(buf), pattern, args);
+	rb_vsnprintf(buf, sizeof(buf), pattern, args);
 	va_end(args);
 
 	if(IsServer(source_p))
-		linebuf_putmsg(&linebuf_local, NULL, NULL,
-			       ":%s %s", source_p->name, buf);
+		rb_linebuf_putmsg(&rb_linebuf_local, NULL, NULL, ":%s %s", source_p->name, buf);
 	else
-		linebuf_putmsg(&linebuf_local, NULL, NULL,
-			       ":%s!%s@%s %s",
-			       source_p->name, source_p->username, 
-			       source_p->host, buf);
+		rb_linebuf_putmsg(&rb_linebuf_local, NULL, NULL,
+				  ":%s!%s@%s %s",
+				  source_p->name, source_p->username, source_p->host, buf);
 
-	linebuf_putmsg(&linebuf_name, NULL, NULL, ":%s %s", source_p->name, buf);
-	linebuf_putmsg(&linebuf_id, NULL, NULL, ":%s %s", use_id(source_p), buf);
+	rb_linebuf_putmsg(&rb_linebuf_name, NULL, NULL, ":%s %s", source_p->name, buf);
+	rb_linebuf_putmsg(&rb_linebuf_id, NULL, NULL, ":%s %s", use_id(source_p), buf);
 
 	if(what == MATCH_HOST)
 	{
-		DLINK_FOREACH_SAFE(ptr, next_ptr, lclient_list.head)
+		RB_DLINK_FOREACH_SAFE(ptr, next_ptr, lclient_list.head)
 		{
 			target_p = ptr->data;
 
 			if(match(mask, target_p->host))
-				_send_linebuf(target_p, &linebuf_local);
+				send_linebuf(target_p, &rb_linebuf_local);
 		}
 	}
 	/* what = MATCH_SERVER, if it doesnt match us, just send remote */
 	else if(match(mask, me.name))
 	{
-		DLINK_FOREACH_SAFE(ptr, next_ptr, lclient_list.head)
+		RB_DLINK_FOREACH_SAFE(ptr, next_ptr, lclient_list.head)
 		{
 			target_p = ptr->data;
-			_send_linebuf(target_p, &linebuf_local);
+			send_linebuf(target_p, &rb_linebuf_local);
 		}
 	}
 
-	DLINK_FOREACH(ptr, serv_list.head)
+	RB_DLINK_FOREACH(ptr, serv_list.head)
 	{
 		target_p = ptr->data;
 
@@ -748,14 +735,14 @@ sendto_match_butone(struct Client *one, struct Client *source_p,
 			continue;
 
 		if(has_id(target_p))
-			send_linebuf_remote(target_p, source_p, &linebuf_id);
+			send_rb_linebuf_remote(target_p, source_p, &rb_linebuf_id);
 		else
-			send_linebuf_remote(target_p, source_p, &linebuf_name);
+			send_rb_linebuf_remote(target_p, source_p, &rb_linebuf_name);
 	}
 
-	linebuf_donebuf(&linebuf_local);
-	linebuf_donebuf(&linebuf_id);
-	linebuf_donebuf(&linebuf_name);
+	rb_linebuf_donebuf(&rb_linebuf_local);
+	rb_linebuf_donebuf(&rb_linebuf_id);
+	rb_linebuf_donebuf(&rb_linebuf_name);
 }
 
 /* sendto_match_servs()
@@ -765,34 +752,32 @@ sendto_match_butone(struct Client *one, struct Client *source_p,
  * side effects - message is sent to matching servers with caps.
  */
 void
-sendto_match_servs(struct Client *source_p, const char *mask, int cap, 
-			int nocap, const char *pattern, ...)
+sendto_match_servs(struct Client *source_p, const char *mask, int cap,
+		   int nocap, const char *pattern, ...)
 {
 	static char buf[BUFSIZE];
 	va_list args;
-	dlink_node *ptr;
+	rb_dlink_node *ptr;
 	struct Client *target_p;
-	buf_head_t linebuf_id;
-	buf_head_t linebuf_name;
+	buf_head_t rb_linebuf_id;
+	buf_head_t rb_linebuf_name;
 
 	if(EmptyString(mask))
 		return;
 
-	linebuf_newbuf(&linebuf_id);
-	linebuf_newbuf(&linebuf_name);
+	rb_linebuf_newbuf(&rb_linebuf_id);
+	rb_linebuf_newbuf(&rb_linebuf_name);
 
 	va_start(args, pattern);
-	ircvsnprintf(buf, sizeof(buf), pattern, args);
+	rb_vsnprintf(buf, sizeof(buf), pattern, args);
 	va_end(args);
 
-	linebuf_putmsg(&linebuf_id, NULL, NULL, 
-			":%s %s", use_id(source_p), buf);
-	linebuf_putmsg(&linebuf_name, NULL, NULL, 
-			":%s %s", source_p->name, buf);
+	rb_linebuf_putmsg(&rb_linebuf_id, NULL, NULL, ":%s %s", use_id(source_p), buf);
+	rb_linebuf_putmsg(&rb_linebuf_name, NULL, NULL, ":%s %s", source_p->name, buf);
 
 	current_serial++;
 
-	DLINK_FOREACH(ptr, global_serv_list.head)
+	RB_DLINK_FOREACH(ptr, global_serv_list.head)
 	{
 		target_p = ptr->data;
 
@@ -817,14 +802,14 @@ sendto_match_servs(struct Client *source_p, const char *mask, int cap,
 				continue;
 
 			if(has_id(target_p->from))
-				_send_linebuf(target_p->from, &linebuf_id);
+				send_linebuf(target_p->from, &rb_linebuf_id);
 			else
-				_send_linebuf(target_p->from, &linebuf_name);
+				send_linebuf(target_p->from, &rb_linebuf_name);
 		}
 	}
 
-	linebuf_donebuf(&linebuf_id);
-	linebuf_donebuf(&linebuf_name);
+	rb_linebuf_donebuf(&rb_linebuf_id);
+	rb_linebuf_donebuf(&rb_linebuf_name);
 }
 
 /* sendto_monitor()
@@ -839,26 +824,26 @@ sendto_monitor(struct monitor *monptr, const char *pattern, ...)
 	va_list args;
 	buf_head_t linebuf;
 	struct Client *target_p;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
-	
-	linebuf_newbuf(&linebuf); 
-	
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
+
+	rb_linebuf_newbuf(&linebuf);
+
 	va_start(args, pattern);
-	linebuf_putmsg(&linebuf, pattern, &args, NULL);
+	rb_linebuf_putmsg(&linebuf, pattern, &args, NULL);
 	va_end(args);
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, monptr->users.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, monptr->users.head)
 	{
 		target_p = ptr->data;
 
 		if(IsIOError(target_p))
 			continue;
 
-		_send_linebuf(target_p, &linebuf);
+		send_linebuf(target_p, &linebuf);
 	}
 
-	linebuf_donebuf(&linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 /* sendto_anywhere()
@@ -868,41 +853,38 @@ sendto_monitor(struct monitor *monptr, const char *pattern, ...)
  * side effects - client is sent message with correct prefix.
  */
 void
-sendto_anywhere(struct Client *target_p, struct Client *source_p, 
+sendto_anywhere(struct Client *target_p, struct Client *source_p,
 		const char *command, const char *pattern, ...)
 {
 	va_list args;
 	buf_head_t linebuf;
 
-	linebuf_newbuf(&linebuf);
+	rb_linebuf_newbuf(&linebuf);
 
 	va_start(args, pattern);
 
 	if(MyClient(target_p))
 	{
 		if(IsServer(source_p))
-			linebuf_putmsg(&linebuf, pattern, &args, ":%s %s %s ",
-				       source_p->name, command, 
-				       target_p->name);
+			rb_linebuf_putmsg(&linebuf, pattern, &args, ":%s %s %s ",
+					  source_p->name, command, target_p->name);
 		else
-			linebuf_putmsg(&linebuf, pattern, &args, 
-				       ":%s!%s@%s %s %s ", 
-				       source_p->name, source_p->username,
-				       source_p->host, command,
-				       target_p->name);
+			rb_linebuf_putmsg(&linebuf, pattern, &args,
+					  ":%s!%s@%s %s %s ",
+					  source_p->name, source_p->username,
+					  source_p->host, command, target_p->name);
 	}
 	else
-		linebuf_putmsg(&linebuf, pattern, &args, ":%s %s %s ",
-			       get_id(source_p, target_p), command,
-			       get_id(target_p, target_p));
+		rb_linebuf_putmsg(&linebuf, pattern, &args, ":%s %s %s ",
+				  get_id(source_p, target_p), command, get_id(target_p, target_p));
 	va_end(args);
 
 	if(MyClient(target_p))
-		_send_linebuf(target_p, &linebuf);
+		send_linebuf(target_p, &linebuf);
 	else
-		send_linebuf_remote(target_p, source_p, &linebuf);
+		send_rb_linebuf_remote(target_p, source_p, &linebuf);
 
-	linebuf_donebuf(&linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 /* sendto_realops_flags()
@@ -915,19 +897,21 @@ void
 sendto_realops_flags(int flags, int level, const char *pattern, ...)
 {
 	struct Client *client_p;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 	va_list args;
 	buf_head_t linebuf;
 
-	linebuf_newbuf(&linebuf);
+	if(EmptyString(me.name))
+		return;
+
+	rb_linebuf_newbuf(&linebuf);
 
 	va_start(args, pattern);
-	linebuf_putmsg(&linebuf, pattern, &args, 
-		       ":%s NOTICE * :*** Notice -- ", me.name);
+	rb_linebuf_putmsg(&linebuf, pattern, &args, ":%s NOTICE * :*** Notice -- ", me.name);
 	va_end(args);
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, oper_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, oper_list.head)
 	{
 		client_p = ptr->data;
 
@@ -939,10 +923,10 @@ sendto_realops_flags(int flags, int level, const char *pattern, ...)
 			continue;
 
 		if(client_p->umodes & flags)
-			_send_linebuf(client_p, &linebuf);
+			send_linebuf(client_p, &linebuf);
 	}
 
-	linebuf_donebuf(&linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 /*
@@ -958,37 +942,32 @@ void
 sendto_wallops_flags(int flags, struct Client *source_p, const char *pattern, ...)
 {
 	struct Client *client_p;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
 	va_list args;
 	buf_head_t linebuf;
 
-	linebuf_newbuf(&linebuf);
+	rb_linebuf_newbuf(&linebuf);
 
 	va_start(args, pattern);
 
-	if(IsPerson(source_p))
-		linebuf_putmsg(&linebuf, pattern, &args,
-			       ":%s!%s@%s WALLOPS :", source_p->name,
-			       source_p->username, source_p->host);
+	if(IsClient(source_p))
+		rb_linebuf_putmsg(&linebuf, pattern, &args,
+				  ":%s!%s@%s WALLOPS :", source_p->name,
+				  source_p->username, source_p->host);
 	else
-		linebuf_putmsg(&linebuf, pattern, &args, ":%s WALLOPS :", source_p->name);
+		rb_linebuf_putmsg(&linebuf, pattern, &args, ":%s WALLOPS :", source_p->name);
 
 	va_end(args);
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, oper_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, oper_list.head)
 	{
 		client_p = ptr->data;
-
-		/* dont allow people without operwall privs to see operwalls */
-		if((flags == UMODE_OPERWALL) && !IsOperOperwall(client_p))
-			continue;
-
 		if(client_p->umodes & flags)
-			_send_linebuf(client_p, &linebuf);
+			send_linebuf(client_p, &linebuf);
 	}
 
-	linebuf_donebuf(&linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 /* kill_client()
@@ -1003,15 +982,15 @@ kill_client(struct Client *target_p, struct Client *diedie, const char *pattern,
 	va_list args;
 	buf_head_t linebuf;
 
-	linebuf_newbuf(&linebuf);
+	rb_linebuf_newbuf(&linebuf);
 
 	va_start(args, pattern);
-	linebuf_putmsg(&linebuf, pattern, &args, ":%s KILL %s :",
-		      get_id(&me, target_p), get_id(diedie, target_p));
+	rb_linebuf_putmsg(&linebuf, pattern, &args, ":%s KILL %s :",
+			  get_id(&me, target_p), get_id(diedie, target_p));
 	va_end(args);
 
-	send_linebuf(target_p, &linebuf);
-	linebuf_donebuf(&linebuf);
+	send_linebuf(target_p->from ? target_p->from : target_p, &linebuf);
+	rb_linebuf_donebuf(&linebuf);
 }
 
 
@@ -1032,24 +1011,24 @@ kill_client_serv_butone(struct Client *one, struct Client *target_p, const char
 	static char buf[BUFSIZE];
 	va_list args;
 	struct Client *client_p;
-	dlink_node *ptr;
-	dlink_node *next_ptr;
-	buf_head_t linebuf_id;
-	buf_head_t linebuf_name;
+	rb_dlink_node *ptr;
+	rb_dlink_node *next_ptr;
+	buf_head_t rb_linebuf_id;
+	buf_head_t rb_linebuf_name;
+
+	rb_linebuf_newbuf(&rb_linebuf_name);
+	rb_linebuf_newbuf(&rb_linebuf_id);
 
-	linebuf_newbuf(&linebuf_name);
-	linebuf_newbuf(&linebuf_id);
-	
 	va_start(args, pattern);
-	ircvsnprintf(buf, sizeof(buf), pattern, args);
+	rb_vsnprintf(buf, sizeof(buf), pattern, args);
 	va_end(args);
 
-	linebuf_putmsg(&linebuf_name, NULL, NULL, ":%s KILL %s :%s",
-		       me.name, target_p->name, buf);
-	linebuf_putmsg(&linebuf_id, NULL, NULL, ":%s KILL %s :%s",
-		       use_id(&me), use_id(target_p), buf);
+	rb_linebuf_putmsg(&rb_linebuf_name, NULL, NULL, ":%s KILL %s :%s",
+			  me.name, target_p->name, buf);
+	rb_linebuf_putmsg(&rb_linebuf_id, NULL, NULL, ":%s KILL %s :%s",
+			  use_id(&me), use_id(target_p), buf);
 
-	DLINK_FOREACH_SAFE(ptr, next_ptr, serv_list.head)
+	RB_DLINK_FOREACH_SAFE(ptr, next_ptr, serv_list.head)
 	{
 		client_p = ptr->data;
 
@@ -1057,15 +1036,15 @@ kill_client_serv_butone(struct Client *one, struct Client *target_p, const char
 		 * ID, then we still want to issue the kill there..
 		 */
 		if(one != NULL && (client_p == one->from) &&
-			(!has_id(client_p) || !has_id(target_p)))
+		   (!has_id(client_p) || !has_id(target_p)))
 			continue;
 
 		if(has_id(client_p))
-			_send_linebuf(client_p, &linebuf_id);
+			send_linebuf(client_p, &rb_linebuf_id);
 		else
-			_send_linebuf(client_p, &linebuf_name);
+			send_linebuf(client_p, &rb_linebuf_name);
 	}
 
-	linebuf_donebuf(&linebuf_id);
-	linebuf_donebuf(&linebuf_name);
+	rb_linebuf_donebuf(&rb_linebuf_id);
+	rb_linebuf_donebuf(&rb_linebuf_name);
 }
